/*****************************************************************************
 * Microchip Graphics Library Hands On Exercise
 * This program shows how to use the Graphics Library from the 
 * Primitive Layer to the Graphics Objects Layer.
 *****************************************************************************
 * FileName:        Lab1.c
 * Dependencies:    MainGFXDemo.h
 * Processor:       PIC24FJ256GA010, PIC24FJ256DA210, PIC32MX360F512L
 * Compiler:       	MPLAB?C Compiler for PIC24 MCUs and dsPIC?DSCs, 
 *					MPLAB?C Compiler for PIC32 MCUs
 * Linker:          MPLAB Link30, MPLAB Link32 
 * Company:         Microchip Technology Incorporated
 *
 * Software License Agreement
 *
 * Copyright ?2010 Microchip Technology Inc.  All rights reserved.
 * Microchip licenses to you the right to use, modify, copy and distribute
 * Software only when embedded on a Microchip microcontroller or digital
 * signal controller, which is integrated into your product or third party
 * product (pursuant to the sublicense terms in the accompanying license
 * agreement).  
 *
 * You should refer to the license agreement accompanying this Software
 * for additional information regarding your rights and obligations.
 *
 * SOFTWARE AND DOCUMENTATION ARE PROVIDED AS IS?WITHOUT WARRANTY OF ANY
 * KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY WARRANTY
 * OF MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR
 * PURPOSE. IN NO EVENT SHALL MICROCHIP OR ITS LICENSORS BE LIABLE OR
 * OBLIGATED UNDER CONTRACT, NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION,
 * BREACH OF WARRANTY, OR OTHER LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT
 * DAMAGES OR EXPENSES INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL,
 * INDIRECT, PUNITIVE OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA,
 * COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY
 * CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF),
 * OR OTHER SIMILAR COSTS.
 *
 * Author               Date        Comment
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Kristy Seymour      04/22/2010    ...
 *****************************************************************************/
#include "MainGFXDemo.h"

///////////////////////////////////////////////////////////////////////////////
////  CONFIGURATION BITS
///////////////////////////////////////////////////////////////////////////////
#if defined(__PIC32MX__)
    #if defined(__32MX360F512L__) || defined (PIC32_GP_SK_DM320001)
        #pragma config FPLLMUL = MUL_20		// PLL Multiplier
        #pragma config FPLLIDIV = DIV_2     // PLL Input Divider
        #pragma config FPLLODIV = DIV_1     // PLL Output Divider
        #pragma config FPBDIV = DIV_2		// Peripheral Clock Divisor
        #pragma config FWDTEN = OFF			// Watchdog Timer Disabled
        #pragma config FCKSM = CSECME		// Clock Switching Enabled & Fail Safe Clock Monitor Enabled
        #pragma config OSCIOFNC = OFF		// CLKO disabled 
        #pragma config POSCMOD = XT			// XT Oscillator Selected
        #pragma config FSOSCEN = ON			// Secondary Oscillator Enabled
        #pragma config FNOSC = PRIPLL		// Primary Oscillator Selection
        #pragma config CP = OFF				// Code Protect Disabled
        #pragma config BWP = OFF			// Boot Flash Write Protect Disabled
        #pragma config PWP = OFF			// Program Flash Write Protect Disabled
    #endif    
#else

    #if defined(__PIC24FJ128GA010__)
_CONFIG2(FNOSC_PRIPLL & POSCMOD_XT) 		// Primary XT OSC with PLL
_CONFIG1(JTAGEN_OFF & FWDTEN_OFF)   		// JTAG off, watchdog timer off
    #endif
	#if defined (__PIC24FJ256DA210__)
_CONFIG1( WDTPS_PS32768 & FWPSA_PR128 & ALTVREF_ALTVREDIS & WINDIS_OFF & FWDTEN_OFF & ICS_PGx2 & GWRP_OFF & GCP_OFF & JTAGEN_OFF) 
_CONFIG2( POSCMOD_HS & IOL1WAY_OFF & OSCIOFNC_OFF & OSCIOFNC_OFF & FCKSM_CSDCMD & FNOSC_PRIPLL & PLL96MHZ_ON & PLLDIV_DIV2 & IESO_OFF)
_CONFIG3( WPFP_WPFP255 & SOSCSEL_SOSC & WUTSEL_LEG & ALTPMP_ALTPMPEN & WPDIS_WPDIS & WPCFG_WPCFGDIS & WPEND_WPENDMEM) 
	#endif	

#endif

////////////////////////////////////////////////////////////////////////////
//                            LOCAL PROTOTYPES
////////////////////////////////////////////////////////////////////////////
void StartScreen(void);										// intro splash screen
void EraseScreen(void);										// splash screen erase
void InitLED(void);											// initialize LED Port
void SetLED(BYTE status, BYTE whichOne); 					// enable/disable LED	
void TickInit(void);                        				// starts tick counter  
void CheckTouchCal(void);									// Check touchscreen calibration

/////////////////////////////////////////////////////////////////////////////
//                            IMAGES USED
/////////////////////////////////////////////////////////////////////////////
extern const BITMAP_FLASH intro;  //### <--- Change image name here ###

/////////////////////////////////////////////////////////////////////////////
//                             FONTS USED
/////////////////////////////////////////////////////////////////////////////
// ### Your Code Here ###		  //### <--- Declare MyNewFont here ###

extern const FONT_FLASH GOLFontDefault; 					// Default GOL font.


/////////////////////////////////////////////////////////////////////////////
//                       GLOBAL VARIABLES FOR DEMO
/////////////////////////////////////////////////////////////////////////////
volatile DWORD tick = 0;                    		 		// tick counter
volatile DWORD prevTick = 0;                    	 		// previous tick count

/////////////////////////////////////////////////////////////////////////////
//                                  MAIN
/////////////////////////////////////////////////////////////////////////////
int main(void){

/* ************************************************************ */ 
//	Device and Hardware Specific Settings
/* ************************************************************ */ 
#ifdef __PIC32MX__
    INTEnableSystemMultiVectoredInt();
    SYSTEMConfigPerformance(GetSystemClock());
#endif

#if defined (PIC24FJ256DA210_DEV_BOARD)
    
     ANSA = 0x0000;
     ANSB = 0x0020;		// RB5 as potentiometer input
     ANSC = 0x0010;		// RC4 as touch screen X+, RC14 as external source of secondary oscillator
     ANSD = 0x0000;
     ANSE = 0x0000;		// RE9 used as PAD1
     ANSF = 0x0000;
     ANSG = 0x0080;		// RG8 used as S1, RG7 as touch screen Y+
        
#else

    /////////////////////////////////////////////////////////////////////////////
    // ADC Explorer 16 Development Board Errata (work around 2)
    // RB15 should be output
    /////////////////////////////////////////////////////////////////////////////
    LATBbits.LATB15 = 0;
    TRISBbits.TRISB15 = 0;

#endif


/* ************************************************************ */ 
//	Initialization Calls
/* ************************************************************ */ 
    GOLInit();      			// Initialize graphics library and create default style scheme for GOL
    SST25Init();   				// Initialize SPI Flash Memory
    TouchInit();    			// Initialize touch screen
    TickInit();     			// Start tick counter    
    InitLED();					// initialize explorer 16 LED
    CheckTouchCal();			// Check touchscreen calibration

	// clear the screen with BLACK color
   	SetColor(BLACK);
    ClearDevice();

    // call the start screen routine
	StartScreen();				// Display the splash screen

	while(1) {}; 
}

/*********************************************************************
* Function: WORD GOLMsgCallback(WORD objMsg, OBJ_HEADER* pObj, 
*								GOL_MSG* pMsg)
*
* Overview: This function must be implemented by user. GOLMsg() 
*			function calls it each time the valid message for 
*			the object received
*
* Input: objMsg - translated message for the object,
*        pObj - pointer to the object,
*        pMsg - pointer to the non-translated, raw GOL message 
*
* Output: if the function returns non-zero the draw control 
*		  will be passed to GOL
*
********************************************************************/
WORD GOLMsgCallback(WORD objMsg, OBJ_HEADER* pObj, GOL_MSG* pMsg) 
{
	return 1;
}	

/*********************************************************************
* Function: WORD GOLDrawCallback()
*
* Overview: This function must be implemented by user. GOLDraw() 
*			function calls it each time when GOL objects drawing is 
*			completed. User drawing should be done here. GOL will 
*			not change color, line type and clipping region settings 
*			while this function returns zero.
*
* Output: if the function returns non-zero the draw control 
*		  will be passed to GOL
*
********************************************************************/
WORD GOLDrawCallback()
{
	return 1;
}	

/*********************************************************************
* Function: void InitLED(void)
*
* Overview: This function initializes PORTA to enable the usage of  
* 			the LED in the Explorer 16 Board. Bit7 is not initialized 
*			since PORTA7 is also used as an input by Switch S5 on the 
*			board.
*
* Output: none
*
********************************************************************/
void InitLED(void)
{

#if defined (PIC24FJ256DA210_DEV_BOARD)
   	TRISEbits.TRISE9  = 0;		// LED D2 to output mode
   	TRISAbits.TRISA7  = 0;		// LED D4 to output mode
   	LATEbits.LATE9    = 0;		// initialize to off 					
   	LATAbits.LATA7    = 0;		// initialize to off 					
#else
   	TRISA = 0xFF00;				// set IO pins (0:6) to output mode, Bit 7 is used with S5
   	LATA = 0x0000;				// initialize all to be off 					
#endif
}	

/*********************************************************************
* Function: void SetLED(BYTE status, BYTE whichOne)
*
* Overview: This function assumes that LEDs connected to PORTA is 
*			initialized as outputs. whichOne is a BYTE that represents 
*			the LEDs where Bit 0 represent D3 and Bit 6 represents D9. 
*			status determines if the LEDs determined by whichOne will 
*			be turned off or on.
*			status:
*			  LED_ON  - turn on LEDs specified by whichOne
*			  LED_OFF - turn off LEDs specified by whichOne
*
* Output: none
*
********************************************************************/
void SetLED(BYTE status, BYTE whichOne) 
{
#if defined (PIC24FJ256DA210_DEV_BOARD)
	if (whichOne & 0x02)
			LATEbits.LATE9 = ((status == LED_OFF) ? 0 : 1);
	if (whichOne & 0x10)
			LATAbits.LATA7 = ((status == LED_OFF) ? 0 : 1);
#else
	if (whichOne & 0x01)
			LATAbits.LATA0 = ((status == LED_OFF) ? 0 : 1);
	if (whichOne & 0x02)
			LATAbits.LATA1 = ((status == LED_OFF) ? 0 : 1);
	if (whichOne & 0x04)
			LATAbits.LATA2 = ((status == LED_OFF) ? 0 : 1);
	if (whichOne & 0x08)
			LATAbits.LATA3 = ((status == LED_OFF) ? 0 : 1);
	if (whichOne & 0x10)
			LATAbits.LATA4 = ((status == LED_OFF) ? 0 : 1);
	if (whichOne & 0x20)
			LATAbits.LATA5 = ((status == LED_OFF) ? 0 : 1);
	if (whichOne & 0x40)
			LATAbits.LATA6 = ((status == LED_OFF) ? 0 : 1);
	if (whichOne & 0x80)
			LATAbits.LATA7 = ((status == LED_OFF) ? 0 : 1);
#endif			
}		

/*********************************************************************
* Function: void StartScreen(void)
*
* Overview: This creates a splash screen start screen using Primitive 
*			functions.
*
* Output: none
*
********************************************************************/
void StartScreen(void) {

#define compileSwitch 0
static const XCHAR text[] 		= "Microchip Graphics Library";
    #if compileSwitch
    static const XCHAR textNext[] = "Lab 1 complete! Way to go!";
    #else
    static const XCHAR textNext[] = "Surprise!!!!";
    #endif
			 SHORT polyPts[8] 	= {95, 155, 139, 199, 95, 199, 95, 155};

	WORD x_image, x_text;
	
	// Lab1: The Primitives - Steps 17-25

	// clear the screen with WHITE color
	//### <--- Insert code to set screen color to WHITE here ###
	SetColor(WHITE);
	ClearDevice();

    PutImage(0,0,(void*)&intro,IMAGE_X2);	
 
 	//### <--- Insert code to change the splash screen image ###
 	//### <--- Add code to calculate x_image ###
    //### <--- Add code display image on the screen ###	

 	//### <--- Insert code to display the string defined by text[] ###
	//### <--- Add code to set color to BRIGHTRED ###			
	//### <--- Add code to set the font  ###
	//### <--- Add code calculate x_text ###
	//### <--- Add code display string  ### 

	// primitive shapes
 	//### <--- Insert code to display the BRIGHTGREEN Bar ###
	//### <--- Add code to set color to BRIGHTGREEN ###
	//### <--- Add code to draw bar ###

 	//### <--- Insert code to display the BRIGHTYELLOW Circle with Normal line size ###
	//### <--- Add code to set color to BRIGHTYELLOW ### 
	//### <--- Add code to set line size to NORMAL_LINE ### 
	//### <--- Add code to draw Circle ###
 
#if compileSwitch	
	SetColor(BRIGHTRED);
	SetLineThickness(THICK_LINE);
	while(!DrawPoly(4, polyPts));

	SetColor(BRIGHTBLUE);
	while(!FillBevel(168, 177, 190, 177, 22));	
	
	// primitive lines
	SetLineThickness(THICK_LINE);
	SetColor(BRIGHTRED);
	while(!Line(44, 205, 263, 205));
	
	SetColor(BRIGHTBLUE);
	SetLineThickness(NORMAL_LINE);
	SetLineType(DOTTED_LINE);
	while(!Line(44, 210, 263, 210));

	SetColor(BRIGHTGREEN);
	SetLineType(DASHED_LINE);
	while(!Line(44, 215, 263, 215));
  
#endif
		
    SetLED(LED_ON, 0xFF);

	// wait for touch
    while((TouchGetY() == -1) && (TouchGetX() == -1)); 
    SetLED(LED_OFF, 0xFF);
	
	EraseScreen();

	// print the message in the middle of the screen 
	SetColor(WHITE);
	SetFont((void*)&GOLFontDefault);
	MoveTo((GetMaxX()-GetTextWidth((XCHAR*)textNext,(void*)&GOLFontDefault))>>1,
		   (GetMaxY()-GetTextHeight((void*)&GOLFontDefault))>>1);
    while(!OutText((XCHAR*)textNext));

}	

/*********************************************************************
* Function: void EraseScreen(void)
*
* Overview: This clears the start splash screen start screen.
*
* Output: none
*
********************************************************************/
void EraseScreen(void) {
	
	SHORT ytemp, xtemp; 
	SHORT i, j;

	// Erase effect routine
    SetColor(BLACK);
    xtemp = GetMaxX()/10;
    ytemp = GetMaxY()/10;
    for (i = 0; i <= (GetMaxX()>>1); i += xtemp) {
    	// by replicating the white (1x1) bars into 8 areas fading is faster
		for (j = 0; j <= GetMaxY() ; j += ytemp) {
			while(!Bar(i, j, i+xtemp, j+ytemp));
			while(!Bar((GetMaxX()-xtemp)-i, (GetMaxY()-j)-ytemp, GetMaxX()-i, (GetMaxY()-j)));
		   	DelayMs(10);
	    }		
	}
	SetColor(WHITE);
	SetLineThickness(THICK_LINE);
    for (i = 0; i <= (GetMaxY()>>1); i += ytemp) {
		SetColor(WHITE);
		while(!Line((GetMaxX()>>1)-(i+xtemp), GetMaxY()>>1, (GetMaxX()>>1)-(i),       GetMaxY()>>1));
		while(!Line((GetMaxX()>>1)+(i),       GetMaxY()>>1, (GetMaxX()>>1)+(i+xtemp), GetMaxY()>>1));
		while(!Line((GetMaxX()>>1), (GetMaxY()>>1)-(i+ytemp), (GetMaxX()>>1), (GetMaxY()>>1)-(i)));
		while(!Line((GetMaxX()>>1), (GetMaxY()>>1)+(i),       (GetMaxX()>>1), (GetMaxY()>>1)+(i+ytemp)));

	   	DelayMs(50);

		SetColor(BLACK);
		while(!Line((GetMaxX()>>1)-(i+xtemp), GetMaxY()>>1, (GetMaxX()>>1)-(i),       GetMaxY()>>1));
		while(!Line((GetMaxX()>>1)+(i),       GetMaxY()>>1, (GetMaxX()>>1)+(i+xtemp), GetMaxY()>>1));
		while(!Line((GetMaxX()>>1), (GetMaxY()>>1)-(i+ytemp), (GetMaxX()>>1), (GetMaxY()>>1)-(i)));
		while(!Line((GetMaxX()>>1), (GetMaxY()>>1)+(i),       (GetMaxX()>>1), (GetMaxY()>>1)+(i+ytemp)));
	}

}	


/*********************************************************************
* Function: Timer4 ISR
*
* PreCondition: none
*
* Input: none
*
* Output: none
*
* Side Effects: none
*
* Overview: increments tick counter. Tick is approx. 1 ms.
*
* Note: none
*
********************************************************************/
#ifdef __PIC32MX__
#define __T4_ISR    __ISR(_TIMER_4_VECTOR, ipl1)
#else
#define __T4_ISR    __attribute__((interrupt, shadow, auto_psv))
#endif

void  __T4_ISR _T4Interrupt(void)
{
    tick++;    
    TouchProcessTouch();
    // Clear flag
#ifdef __PIC32MX__
    mT4ClearIntFlag();
#else
    IFS1bits.T4IF = 0;
#endif
}

/*********************************************************************
* Function: void TickInit(void)
*
* PreCondition: none
*
* Input: none
*
* Output: none
*
* Side Effects: none
*
* Overview: sets tick timer
*
* Note: none
*
********************************************************************/
/*********************************************************************
 * Section: Tick Delay
 *********************************************************************/
#define SAMPLE_PERIOD       500 // us
#define TICK_PERIOD			(GetPeripheralClock() * SAMPLE_PERIOD) / 4000000

void TickInit(void){
    // Initialize Timer4
#ifdef __PIC32MX__
    OpenTimer4(T4_ON | T4_PS_1_8, TICK_PERIOD);
    ConfigIntTimer4(T4_INT_ON | T4_INT_PRIOR_1);
#else
    TMR4 = 0;
    PR4 = TICK_PERIOD;
    IFS1bits.T4IF = 0;              //Clear flag
    IEC1bits.T4IE = 1;              //Enable interrupt
    T4CONbits.TON = 1;              //Run timer  
#endif
}



void CheckTouchCal(void){

#if defined (PIC24FJ256DA210_DEV_BOARD)
	// If S1 button on PIC24FJ256DA210 Development Board is pressed calibrate touch screen
    if(PORTGbits.RG8 == 0){
#else
	// If S3 button on Explorer 16 board is pressed calibrate touch screen
    if(PORTDbits.RD6 == 0){
#endif	    
        TouchCalibration();
        TouchStoreCalibration();
    } else {
   		if(GRAPHICS_LIBRARY_VERSION != SST25ReadWord(ADDRESS_VERSION)){
       		TouchCalibration();
       		TouchStoreCalibration();
   		}
	
	    // Load touch screen calibration parameters from EEPROM
	    TouchLoadCalibration();
	}    
}



