/*****************************************************************************
* Microchip Graphics Library
* Graphics Display Designer (GDD) Template
*****************************************************************************
* Dependencies:    See INCLUDES section below
* Processor:       PIC24F, PIC24H, dsPIC, PIC32
* Compiler:       	MPLAB C30 V3.22, MPLAB C32 V1.05b
* Linker:          MPLAB LINK30, MPLAB LINK32
* Company:         Microchip Technology Incorporated
*
* Software License Agreement
*
* Copyright (c) 2010 Microchip Technology Inc.  All rights reserved.
* Microchip licenses to you the right to use, modify, copy and distribute
* Software only when embedded on a Microchip microcontroller or digital
* signal controller, which is integrated into your product or third party
* product (pursuant to the sublicense terms in the accompanying license
* agreement).  
*
* You should refer to the license agreement accompanying this Software
* for additional information regarding your rights and obligations.
*
* SOFTWARE AND DOCUMENTATION ARE PROVIDED AS IS WITHOUT WARRANTY OF ANY
* KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY WARRANTY
* OF MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR
* PURPOSE. IN NO EVENT SHALL MICROCHIP OR ITS LICENSORS BE LIABLE OR
* OBLIGATED UNDER CONTRACT, NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION,
* BREACH OF WARRANTY, OR OTHER LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT
* DAMAGES OR EXPENSES INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL,
* INDIRECT, PUNITIVE OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA,
* COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY
* CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF),
* OR OTHER SIMILAR COSTS.
*
* Author               Date        Comment
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*Kristy Seymour		4/20/10		Generated by GDD Beta version2.0.1
*								Modified to suit demo application
*****************************************************************************/
#include "Graphics\Graphics.h"
#include "MainGFXDemo.h"

// Strings
GOL_SCHEME* statusbox;
GOL_SCHEME* flatbuttons;
GOL_SCHEME* labelbox;





//  Create Error Flash (Blue Screen)
void CreateError(char* string)
{
	// Blue Screen Error
	SetColor(119);
	ClearDevice();
	SetColor(-1);
	
	// Flash Error Message
	if(string == NULL)
		{OutTextXY(0, 0, "Runtime Error.");}
	else{OutTextXY(0,0, string);}
	while(1);
}

void CreateSchemes(void)
{
	// free the memory used by the style schemes in case they exist
	// these are catch all code to make sure we do not clutter the heap
	if(statusbox   != NULL) free(statusbox);
	if(labelbox    != NULL) free(labelbox);
	if(flatbuttons != NULL) free(flatbuttons);

	// create the style schemes
	//### <--- Lab2 Step 6: Add code here to create the two static text style schemes ###
	statusbox = GOLCreateScheme();
	labelbox = GOLCreateScheme();
		
	// Assign colors to the style scheme created

#if 1	//### <--- Lab2 Step 7: Set this compile switch to 1 to enable the modification of the style scheme ###

	// style scheme for the Static Text used for the title
	statusbox->Color0 = BLACK;
	statusbox->Color1 = 64969;
	statusbox->ColorDisabled = 55070;
	statusbox->CommonBkColor = SCREEN_BACKGROUND_COLOR;
	statusbox->EmbossDkColor = BLACK;
	statusbox->EmbossLtColor = BLACK;
	statusbox->TextColor0 = 65535;
	statusbox->TextColor1 = 31;
	statusbox->pFont = (void*)&MyNewFont;

	// style scheme for the labels of the buttons/icons 
	labelbox->Color0 = BLACK;
	labelbox->Color1 = 64969;
	labelbox->ColorDisabled = 55070;
	labelbox->CommonBkColor = SCREEN_BACKGROUND_COLOR;
	labelbox->EmbossDkColor = BLACK;
	labelbox->EmbossLtColor = BLACK;
	labelbox->TextColor0 = 65535;
	labelbox->TextColor1 = 31;
	labelbox->pFont = (void*)&SmallFonts;

#endif

	// style scheme for the buttons/icons
	flatbuttons = GOLCreateScheme();
	
#if 1	//### <--- Lab2 Step 11: Set this compile switch to 1 to enable the modification of the style scheme ###

	flatbuttons->Color0 = RGB565CONVERT(0x75, 0x76, 0x75);
	flatbuttons->Color1 = 64969;
	flatbuttons->ColorDisabled = 55070;
	flatbuttons->CommonBkColor = SCREEN_BACKGROUND_COLOR;
	flatbuttons->EmbossDkColor = RGB565CONVERT(0x55, 0x56, 0x55); 
	flatbuttons->EmbossLtColor = RGB565CONVERT(0xCE, 0xCF, 0xCE);
	flatbuttons->TextColor0 = BRIGHTRED; //8;
	flatbuttons->TextColor1 = WHITE; //31;
	flatbuttons->pFont = (void*)&SmallFonts;
#endif
		
}


void CreateMenuScreen(void)
{

#if defined (USE_BUTTON) 
	// pointers to widgets on this screen
	BUTTON *pOBJ_BUTTON_POT = NULL;
	BUTTON *pOBJ_BUTTON_TEMP = NULL;
	BUTTON *pOBJ_BUTTON_LED = NULL;
#endif
#if defined (USE_STATICTEXT) 
	STATICTEXT *pOBJ_STATICTEXT_TITLE = NULL;
	STATICTEXT *pOBJ_STATICTEXT_POT = NULL;
	STATICTEXT *pOBJ_STATICTEXT_TEMP = NULL;
	STATICTEXT *pOBJ_STATICTEXT_LED = NULL;
#endif	
	static XCHAR Btn_Label[] = "LED"; 
	
	GOLFree();
	SetColor(SCREEN_BACKGROUND_COLOR);
	ClearDevice();



	// Button to launch the potentiometer measurement screen
    //### <--- Lab2 Step 8: Insert code here create Button 1 ###
	pOBJ_BUTTON_POT = BtnCreate(OBJ_BUTTON_ROTATE_POT,
								POT_STARTX,
								POT_STARTY,
								POT_STARTX+BTN_WIDTH,
								POT_STARTY+BTN_HEIGHT,
								BTN_RADIUS,
								BTN_DRAW,
								(void*)&Rotate_Icon,
								NULL,
								NULL);
								
	// Button to launch the temperature monitoring screen
    //### <--- Lab2 Step 8: Insert code here create Button 2 ###
	pOBJ_BUTTON_TEMP = BtnCreate(OBJ_BUTTON_CHECK_TEMP,
								TEMP_STARTX,
								TEMP_STARTY,
								TEMP_STARTX+BTN_WIDTH,
								TEMP_STARTY+BTN_HEIGHT,
								BTN_RADIUS,
								BTN_DRAW,
								(void*)&Temp_Icon,
								NULL,
								NULL);

	// Button to launch LED Screen
    //### <--- Lab2 Step 8: Insert code here create Button 3 ###
	pOBJ_BUTTON_LED = BtnCreate(OBJ_BUTTON_CONTROL_LED,
								LED_STARTX,
								LED_STARTY,
								LED_STARTX+BTN_WIDTH,
								LED_STARTY+BTN_HEIGHT,
								BTN_RADIUS,
								BTN_DRAW,
								NULL, 
								(XCHAR*)Btn_Label,
								NULL);

#if defined (USE_USE_BUTTON) 
	// Error Checks
	if(pOBJ_BUTTON_POT==NULL){ //Fatal Error, Check for memory leak or heap size
		CreateError(0);
	}
	if(pOBJ_BUTTON_TEMP==NULL){//Fatal Error, Check for memory leak or heap size
		CreateError(0);
	}
	if(pOBJ_BUTTON_LED==NULL){//Fatal Error, Check for memory leak or heap size
		CreateError(0);
	}
#endif
    //### <--- Lab2 Step 12: Modify the style scheme used by the 
    //                       pOBJ_BUTTON_LED as specified in the Lab Manuals ###
	GOLSetScheme((OBJ_HEADER*)pOBJ_BUTTON_LED, flatbuttons);

	BtnSetText((BUTTON*)pOBJ_BUTTON_LED, (XCHAR*)Btn_Label);		// added to recalculate the width and height of the string


	// Static Text to label the Potentiometer button
    //### <--- Lab2 Step 13: Insert code here create Static Text 1 ###
	pOBJ_STATICTEXT_POT = StCreate(OBJ_STATICTEXT_POT,
								POT_STARTX,
								POT_STARTY + BTN_HEIGHT + 5,
								POT_STARTX + BTN_WIDTH,
								POT_STARTY + (BTN_HEIGHT*2) + 5,
								ST_CENTER_ALIGN|ST_DRAW,
								"ROTATE\nPOT",
								labelbox);
	
	// Static Text to label the Temperature button
    //### <--- Lab2 Step 13: Insert code here create Static Text 2 ###
	pOBJ_STATICTEXT_TEMP = StCreate(OBJ_STATICTEXT_TEMP,
								TEMP_STARTX,
								TEMP_STARTY + BTN_HEIGHT + 5,
								TEMP_STARTX + BTN_WIDTH,
								TEMP_STARTY + (BTN_HEIGHT*2) + 5,
								ST_CENTER_ALIGN|ST_DRAW,
								"CHECK\nTEMP",
								labelbox);
	
	// Static Text to label the Temperature button
    //### <--- Lab2 Step 13: Insert code here create Static Text 3 ###
	pOBJ_STATICTEXT_LED = StCreate(OBJ_STATICTEXT_LED,
								LED_STARTX,
								LED_STARTY + BTN_HEIGHT + 5,
								LED_STARTX + BTN_WIDTH,
								LED_STARTY + (BTN_HEIGHT*2) + 5,
								ST_CENTER_ALIGN|ST_DRAW,
								"LED\nCONTROL",
								labelbox);

#if 1	    //### <--- Lab2 Step 14: Enable the compile switch to create the title static text ###

	// Static Text to label the screen
	pOBJ_STATICTEXT_TITLE = StCreate(OBJ_STATICTEXT_MAIN,
								ST_STARTX,
								ST_STARTY,
								ST_STARTX+ST_WIDTH,
								ST_STARTY+ST_HEIGHT,
								ST_CENTER_ALIGN|ST_DRAW|ST_FRAME,
								"Explorer 16 Board Status",
								statusbox);

	
	// Error Checks
	if(pOBJ_STATICTEXT_POT==NULL){//Fatal Error, Check for memory leak or heap size
		CreateError(0);
	}
	if(pOBJ_STATICTEXT_TEMP==NULL){//Fatal Error, Check for memory leak or heap size
		CreateError(0);
	}
	if(pOBJ_STATICTEXT_TEMP==NULL){//Fatal Error, Check for memory leak or heap size
		CreateError(0);
	}
	if(pOBJ_STATICTEXT_TITLE==NULL){//Fatal Error, Check for memory leak or heap size
		CreateError(0);
	}
#endif
}

