/*****************************************************************************
 * Microchip Graphics Library Hands On Exercise
 * This program shows how to use the Graphics Library from the 
 * Primitive Layer to the Graphics Objects Layer.
 *****************************************************************************
 * FileName:        MainGFXDemo.c
 * Dependencies:    MainGFXDemo.h
 * Processor:       PIC32MX460F512L
 * Compiler:       	MPLAB C32
 * Linker:          MPLAB LINK32
 * Company:         Microchip Technology Incorporated
 *
 * Software License Agreement
 *
 * Copyright  2008 Microchip Technology Inc.  All rights reserved.
 * Microchip licenses to you the right to use, modify, copy and distribute
 * Software only when embedded on a Microchip microcontroller or digital
 * signal controller, which is integrated into your product or third party
 * product (pursuant to the sublicense terms in the accompanying license
 * agreement).  
 *
 * You should refer to the license agreement accompanying this Software
 * for additional information regarding your rights and obligations.
 *
 * SOFTWARE AND DOCUMENTATION ARE PROVIDED AS IS WITHOUT WARRANTY OF ANY
 * KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY WARRANTY
 * OF MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR
 * PURPOSE. IN NO EVENT SHALL MICROCHIP OR ITS LICENSORS BE LIABLE OR
 * OBLIGATED UNDER CONTRACT, NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION,
 * BREACH OF WARRANTY, OR OTHER LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT
 * DAMAGES OR EXPENSES INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL,
 * INDIRECT, PUNITIVE OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA,
 * COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY
 * CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF),
 * OR OTHER SIMILAR COSTS.
 *
 * Author               Date        Comment
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Kristy Seymour      04/22/2010    ...
 *****************************************************************************/

#include "MainGFXDemo.h"

///////////////////////////////////////////////////////////////////////////////
////  CONFIGURATION BITS
///////////////////////////////////////////////////////////////////////////////
#if defined(__PIC32MX__)
    #if defined(__32MX360F512L__) || defined (PIC32_GP_SK_DM320001)
        #pragma config FPLLMUL = MUL_20		// PLL Multiplier
        #pragma config FPLLIDIV = DIV_2     // PLL Input Divider
        #pragma config FPLLODIV = DIV_1     // PLL Output Divider
        #pragma config FPBDIV = DIV_2		// Peripheral Clock Divisor
        #pragma config FWDTEN = OFF			// Watchdog Timer Disabled
        #pragma config FCKSM = CSECME		// Clock Switching Enabled & Fail Safe Clock Monitor Enabled
        #pragma config OSCIOFNC = OFF		// CLKO disabled 
        #pragma config POSCMOD = XT			// XT Oscillator Selected
        #pragma config FSOSCEN = ON			// Secondary Oscillator Enabled
        #pragma config FNOSC = PRIPLL		// Primary Oscillator Selection
        #pragma config CP = OFF				// Code Protect Disabled
        #pragma config BWP = OFF			// Boot Flash Write Protect Disabled
        #pragma config PWP = OFF			// Program Flash Write Protect Disabled
    #endif    
#else

   #if defined(__PIC24FJ256GB110__)
_CONFIG1(JTAGEN_OFF & GCP_OFF & GWRP_OFF & COE_OFF & FWDTEN_OFF & ICS_PGx2)
_CONFIG2(0xF7FF & IESO_OFF & FCKSM_CSDCMD & OSCIOFNC_OFF & POSCMOD_HS & FNOSC_PRIPLL & PLLDIV_DIV2 & IOL1WAY_OFF)
    #endif
    #if defined(__PIC24FJ256GA110__)
_CONFIG1(JTAGEN_OFF & GCP_OFF & GWRP_OFF & COE_OFF & FWDTEN_OFF & ICS_PGx2)
_CONFIG2(IESO_OFF & FCKSM_CSDCMD & OSCIOFNC_OFF & POSCMOD_HS & FNOSC_PRIPLL & IOL1WAY_OFF)
    #endif
    #if defined(__PIC24FJ128GA010__)
_CONFIG2(FNOSC_PRIPLL & POSCMOD_XT) // Primary XT OSC with PLL
_CONFIG1(JTAGEN_OFF & FWDTEN_OFF)   // JTAG off, watchdog timer off
    #endif
	#if defined (__PIC24FJ256GB210__)
_CONFIG1( WDTPS_PS32768 & FWPSA_PR128 & ALTVREF_ALTVREDIS & WINDIS_OFF & FWDTEN_OFF & ICS_PGx2 & GWRP_OFF & GCP_OFF & JTAGEN_OFF) 
_CONFIG2( POSCMOD_HS & IOL1WAY_OFF & OSCIOFNC_OFF & OSCIOFNC_OFF & FCKSM_CSDCMD & FNOSC_PRIPLL & PLL96MHZ_ON & PLLDIV_DIV2 & IESO_OFF)
_CONFIG3( WPFP_WPFP255 & SOSCSEL_SOSC & WUTSEL_LEG & WPDIS_WPDIS & WPCFG_WPCFGDIS & WPEND_WPENDMEM) 
	#endif
	#if defined (__PIC24FJ256DA210__)
_CONFIG1( WDTPS_PS32768 & FWPSA_PR128 & ALTVREF_ALTVREDIS & WINDIS_OFF & FWDTEN_OFF & ICS_PGx2 & GWRP_OFF & GCP_OFF & JTAGEN_OFF) 
_CONFIG2( POSCMOD_HS & IOL1WAY_OFF & OSCIOFNC_OFF & OSCIOFNC_OFF & FCKSM_CSDCMD & FNOSC_PRIPLL & PLL96MHZ_ON & PLLDIV_DIV2 & IESO_OFF)
_CONFIG3( WPFP_WPFP255 & SOSCSEL_SOSC & WUTSEL_LEG & ALTPMP_ALTPMPEN & WPDIS_WPDIS & WPCFG_WPCFGDIS & WPEND_WPENDMEM) 
	#endif	

#endif

////////////////////////////////////////////////////////////////////////////
//                            LOCAL PROTOTYPES
////////////////////////////////////////////////////////////////////////////
void StartScreen(void);										// intro splash screen
void EraseScreen(void);										// splash screen erase
void InitLED(void);											// initialize LED Port
void SetLED(BYTE status, BYTE whichOne); 					// enable/disable LED	
void TickInit(void);                        				// starts tick counter  
void CheckTouchCal(void);									// Check touchscreen calibration

// Prototypes for screen callbacks
WORD MainMenuMsgCallback(WORD objMsg, OBJ_HEADER* pObj); 	// processes messages for main menu screen
WORD LEDMsgCallback(WORD objMsg, OBJ_HEADER* pObj); 	
WORD PotentiometerMsgCallback(WORD objMsg, OBJ_HEADER* pObj);
WORD TemperatureMsgCallback(WORD objMsg, OBJ_HEADER* pObj);


/////////////////////////////////////////////////////////////////////////////
//                            IMAGES USED
/////////////////////////////////////////////////////////////////////////////
extern const BITMAP_FLASH intro;
extern const BITMAP_FLASH mchpLogo;

/////////////////////////////////////////////////////////////////////////////
//                             FONTS USED
/////////////////////////////////////////////////////////////////////////////
extern const FONT_FLASH MyNewFont;

/////////////////////////////////////////////////////////////////////////////
//                       GLOBAL VARIABLES FOR DEMO
/////////////////////////////////////////////////////////////////////////////
volatile DWORD tick = 0;                    		 		// tick counter
volatile DWORD prevTick = 0;                    	 		// previous tick count
SCREEN_STATES screenState = CREATE_MAINMENU; 	 			// initialize state of main demo state mashine 


/////////////////////////////////////////////////////////////////////////////
//                                  MAIN
/////////////////////////////////////////////////////////////////////////////
int main(void){

	GOL_MSG msg;        			// GOL message structure to interact with GOL

/* ************************************************************ */ 
//	Device and Hardware Specific Settings
/* ************************************************************ */ 
#ifdef __PIC32MX__
    INTEnableSystemMultiVectoredInt();
    SYSTEMConfigPerformance(GetSystemClock());
    DDPCONbits.JTAGEN = 0;			// to disable the JTAG mode on PORT A pins.
#endif

#if defined (PIC24FJ256DA210_DEV_BOARD)
    
     ANSA = 0x0000;
     ANSB = 0x0020;		// RB5 as potentiometer input
     ANSC = 0x0010;		// RC4 as touch screen X+, RC14 as external source of secondary oscillator
     ANSD = 0x0000;
     ANSE = 0x0000;		// RE9 used as S2
     ANSF = 0x0000;
     ANSG = 0x0080;		// RG8 used as S1, RG7 as touch screen Y+
        
#else

    /////////////////////////////////////////////////////////////////////////////
    // ADC Explorer 16 Development Board Errata (work around 2)
    // RB15 should be output
    /////////////////////////////////////////////////////////////////////////////
    LATBbits.LATB15 = 0;
    TRISBbits.TRISB15 = 0;

#endif


/* ************************************************************ */ 
//	Initialization Calls
/* ************************************************************ */ 
    GOLInit();      			// Initialize graphics library and create default style scheme for GOL
    SST25Init();   				// Initialize SPI Flash Memory
    TouchInit();    			// Initialize touch screen
    TickInit();     			// Start tick counter    
    InitLED();					// initialize explorer 16 LED
    CheckTouchCal();			// Check touchscreen calibration

	// clear the screen with WHITE color
   	SetColor(WHITE);
    ClearDevice();

	GOLFree();					// Clear GOL Heap
	CreateSchemes();			// Set up style schemes
	CreateLEDScreen();			// Create the LED control screen
	
	// Main Application Loop  
	while(1) 
	{
		if(GOLDraw())            // Draw GOL objects
        {
	        //### <--- Lab3 Step 1: Add the function call to get the touch message from the touch screen module ###
	        TouchGetMsg(&msg);   // Get message from touch screen
	        
	        //### <--- Lab3 Step 2: Add the function call to process the message by the message manager ###
            GOLMsg(&msg);        // Process message
        }		
	} 
}

/*********************************************************************
* Function: WORD GOLMsgCallback(WORD objMsg, OBJ_HEADER* pObj, 
*								GOL_MSG* pMsg)
*
* Overview: This function must be implemented by user. GOLMsg() 
*			function calls it each time the valid message for 
*			the object received
*
* Input: objMsg - translated message for the object,
*        pObj - pointer to the object,
*        pMsg - pointer to the non-translated, raw GOL message 
*
* Output: if the function returns non-zero the draw control 
*		  will be passed to GOL
*
********************************************************************/
WORD GOLMsgCallback(WORD objMsg, OBJ_HEADER* pObj, GOL_MSG* pMsg) 
{
	WORD whichLED = 0;
	
	switch(GetObjID(pObj)){

#if defined (PIC24FJ256DA210_DEV_BOARD)

		case OBJ_CHECKBOX_2:
			whichLED = 2;				// Turn on LED D2	
            break; 							

		case OBJ_CHECKBOX_4:
			whichLED = 4;				// Turn on LED D4	
            break; 							

#else

		case OBJ_CHECKBOX_1:
			whichLED = 3;				// Turn on LED D3
            break; 							

		case OBJ_CHECKBOX_2:
			whichLED = 4;				// Turn on LED D4	
            break; 							

		case OBJ_CHECKBOX_3:
			whichLED = 5;				// Turn on LED D5	
            break; 							

		case OBJ_CHECKBOX_4:
			whichLED = 6;				// Turn on LED D6	
            break; 							

		case OBJ_CHECKBOX_5:
			whichLED = 7;				// Turn on LED D7	
            break; 							

		case OBJ_CHECKBOX_6:
			whichLED = 8;				// Turn on LED D8	
            break; 							

		case OBJ_CHECKBOX_7:
			whichLED = 9;				// Turn on LED D9	
            break; 							

		case OBJ_CHECKBOX_8:
			whichLED = 10;				// Turn on LED D10	
            break; 							
#endif

        default:
            break; 							
    }
	if(objMsg == CB_MSG_CHECKED)
	{

	    //### <--- Lab3 Steps 5: Add code here to turn ON an LED based on the state of the check box ###
		SetLED(LED_ON, whichLED);		
        //HINT:  Use predefined function, SetLED.  The prototype is:
        //       void SetLED(BYTE status, BYTE whichOne) -- see lab manual pg 3-4 for more detail.

	    //### <--- Lab3 Steps 8: Add code here modify the static text string to display "LED turned on" ###
		StSetText((STATICTEXT*)GOLFindObject(OBJ_STATICTEXT_EVENTS), "LED turned on");
		SetState((STATICTEXT*)GOLFindObject(OBJ_STATICTEXT_EVENTS), ST_DRAW);
       //HINT:  StSetText((STATICTEXT*)GOLFindObject(OBJ_STATICTEXT_EVENTS),"text");  
        //       You must also set the draw bit using the SetStat function
        //       See lab manual pg 3-5 for more details
	}	
	if(objMsg == CB_MSG_UNCHECKED)
	{	
	    //### <--- Lab3 Steps 6: Add code here to turn OFF an LED based on the state of the check box ###
		SetLED(LED_OFF, whichLED);		
        //HINT:  Use predefined function, SetLED.  The prototype is:
        //       void SetLED(BYTE status, BYTE whichOne) -- see lab manual pg 3-4 for more detail.

	    //### <--- Lab3 Steps 9: Add code here modify the static text string to display "LED turned off" ###
		StSetText((STATICTEXT*)GOLFindObject(OBJ_STATICTEXT_EVENTS), "LED turned off");
		SetState((STATICTEXT*)GOLFindObject(OBJ_STATICTEXT_EVENTS), ST_DRAW);
       //HINT:  StSetText((STATICTEXT*)GOLFindObject(OBJ_STATICTEXT_EVENTS),"text");  
        //       You must also set the draw bit using the SetStat function
        //       See lab manual pg 3-5 for more details	}
    }
    return 1;
}	

/*********************************************************************
* Function: WORD GOLDrawCallback()
*
* Overview: This function must be implemented by user. GOLDraw() 
*			function calls it each time when GOL objects drawing is 
*			completed. User drawing should be done here. GOL will 
*			not change color, line type and clipping region settings 
*			while this function returns zero.
*
* Output: if the function returns non-zero the draw control 
*		  will be passed to GOL
*
********************************************************************/
WORD GOLDrawCallback()
{
	return 1;	
}	

/*********************************************************************
* Function: void InitLED(void)
*
* Overview: This function initializes PORTA to enable the usage of  
* 			the LED in the Explorer 16 Board. Bit7 is not initialized 
*			since PORTA7 is also used as an input by Switch S5 on the 
*			board.
*
* Output: none
*
********************************************************************/
void InitLED(void)
{

#if defined (PIC24FJ256DA210_DEV_BOARD)
   	TRISEbits.TRISE9  = 0;		// LED D2 to output mode
   	TRISAbits.TRISA7  = 0;		// LED D4 to output mode
   	LATEbits.LATE9    = 0;		// initialize to off 					
   	LATAbits.LATA7    = 0;		// initialize to off 					
#else
   	TRISA = 0xFF00;				// set IO pins (0:6) to output mode, Bit 7 is used with S5
   	LATA = 0x0000;				// initialize all to be off 					
#endif
}	

/*********************************************************************
* Function: void SetLED(BYTE status, BYTE whichOne)
*
* Overview: This function sets the LED on the Explorer 16 or PIC24FJ256DA210
*			Development Board. whichOne specifies which LED turn off or on.
*			status:
*			  LED_ON  - turn on LEDs specified by whichOne
*			  LED_OFF - turn off LEDs specified by whichOne
*
* Output: none
*
********************************************************************/
void SetLED(BYTE status, BYTE whichOne) 
{

	switch (whichOne)
	{
#if defined (PIC24FJ256DA210_DEV_BOARD)
		case 2:
			LATEbits.LATE9 = ((status == LED_OFF) ? 0 : 1);
			break;
#endif			
#if defined (EXPLORER_16)
		case 3:
			LATAbits.LATA0 = ((status == LED_OFF) ? 0 : 1);
			break;
#endif			
		case 4:
#if defined (PIC24FJ256DA210_DEV_BOARD)
			LATAbits.LATA7 = ((status == LED_OFF) ? 0 : 1);
#else			
			LATAbits.LATA1 = ((status == LED_OFF) ? 0 : 1);
#endif			
			break;
#if defined (EXPLORER_16)
		case 5:
			LATAbits.LATA2 = ((status == LED_OFF) ? 0 : 1);
			break;
		case 6:
			LATAbits.LATA3 = ((status == LED_OFF) ? 0 : 1);
			break;
		case 7:
			LATAbits.LATA4 = ((status == LED_OFF) ? 0 : 1);
			break;
		case 8:
			LATAbits.LATA5 = ((status == LED_OFF) ? 0 : 1);
			break;
		case 9:
			LATAbits.LATA6 = ((status == LED_OFF) ? 0 : 1);
			break;
		case 10:
			LATAbits.LATA7 = ((status == LED_OFF) ? 0 : 1);
			break;
#endif			
		default:
			break;	
	}	
}		

/*********************************************************************
* Function: Timer4 ISR
*
* PreCondition: none
*
* Input: none
*
* Output: none
*
* Side Effects: none
*
* Overview: increments tick counter. Tick is approx. 1 ms.
*
* Note: none
*
********************************************************************/
#ifdef __PIC32MX__
#define __T4_ISR    __ISR(_TIMER_4_VECTOR, ipl1)
#else
#define __T4_ISR    __attribute__((interrupt, shadow, auto_psv))
#endif

void  __T4_ISR _T4Interrupt(void)
{
    tick++;    
    TouchProcessTouch();
    // Clear flag
#ifdef __PIC32MX__
    mT4ClearIntFlag();
#else
    IFS1bits.T4IF = 0;
#endif
}

/*********************************************************************
* Function: void TickInit(void)
*
* PreCondition: none
*
* Input: none
*
* Output: none
*
* Side Effects: none
*
* Overview: sets tick timer
*
* Note: none
*
********************************************************************/
/*********************************************************************
 * Section: Tick Delay
 *********************************************************************/
#define SAMPLE_PERIOD       500 // us
#define TICK_PERIOD			(GetPeripheralClock() * SAMPLE_PERIOD) / 4000000

void TickInit(void){
    // Initialize Timer4
#ifdef __PIC32MX__
    OpenTimer4(T4_ON | T4_PS_1_8, TICK_PERIOD);
    ConfigIntTimer4(T4_INT_ON | T4_INT_PRIOR_1);
#else
    TMR4 = 0;
    PR4 = TICK_PERIOD;
    IFS1bits.T4IF = 0;              //Clear flag
    IEC1bits.T4IE = 1;              //Enable interrupt
    T4CONbits.TON = 1;              //Run timer  
#endif
}



void CheckTouchCal(void){

#if defined (PIC24FJ256DA210_DEV_BOARD)
	// If S1 button on PIC24FJ256DA210 Development Board is pressed calibrate touch screen
    if(PORTGbits.RG8 == 0){
#else
	// If S3 button on Explorer 16 board is pressed calibrate touch screen
    if(PORTDbits.RD6 == 0){
#endif	    
        TouchCalibration();
        TouchStoreCalibration();
    } else {
   		if(GRAPHICS_LIBRARY_VERSION != SST25ReadWord(ADDRESS_VERSION)){
       		TouchCalibration();
       		TouchStoreCalibration();
   		}
	
	    // Load touch screen calibration parameters from EEPROM
	    TouchLoadCalibration();
	}    
}



