/*****************************************************************************
* Microchip Graphics Library
* Graphics Display Designer (GDD) Template
*****************************************************************************
* Dependencies:    See INCLUDES section below
* Processor:       PIC24F, PIC24H, dsPIC, PIC32
* Compiler:       	MPLAB C30 V3.22, MPLAB C32 V1.05b
* Linker:          MPLAB LINK30, MPLAB LINK32
* Company:         Microchip Technology Incorporated
*
* Software License Agreement
*
* Copyright (c) 2010 Microchip Technology Inc.  All rights reserved.
* Microchip licenses to you the right to use, modify, copy and distribute
* Software only when embedded on a Microchip microcontroller or digital
* signal controller, which is integrated into your product or third party
* product (pursuant to the sublicense terms in the accompanying license
* agreement).  
*
* You should refer to the license agreement accompanying this Software
* for additional information regarding your rights and obligations.
*
* SOFTWARE AND DOCUMENTATION ARE PROVIDED AS IS WITHOUT WARRANTY OF ANY
* KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY WARRANTY
* OF MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR
* PURPOSE. IN NO EVENT SHALL MICROCHIP OR ITS LICENSORS BE LIABLE OR
* OBLIGATED UNDER CONTRACT, NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION,
* BREACH OF WARRANTY, OR OTHER LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT
* DAMAGES OR EXPENSES INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL,
* INDIRECT, PUNITIVE OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA,
* COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY
* CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF),
* OR OTHER SIMILAR COSTS.
*
* Author               Date        Comment
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*Kristy Seymour		4/20/10		Generated by GDD Beta version2.0.1
*								Modified to suit demo application
*****************************************************************************/
#include "Graphics\Graphics.h"
#include "MainGFXDemo.h"

// Strings
GOL_SCHEME* checkboxes;
GOL_SCHEME* statusbox;

//  Create Error Flash (Blue Screen)
void CreateError(char* string)
{
	// Blue Screen Error
	SetColor(119);
	ClearDevice();
	SetColor(-1);
	
	// Flash Error Message
	if(string == NULL)
		{OutTextXY(0, 0, "Runtime Error.");}
	else{OutTextXY(0,0, string);}
	while(1);
}

void CreateSchemes(void)
{

	// free the memory used by the style schemes in case they exist
	// these are catch all code to make sure we do not clutter the heap
	if(statusbox   != NULL) free(statusbox);
	if(checkboxes  != NULL) free(checkboxes);
	
	// style scheme for the Static Text used for the title
	statusbox = GOLCreateScheme();
	statusbox->Color0 = BLACK;
	statusbox->Color1 = 64969;
	statusbox->ColorDisabled = 55070;
	statusbox->CommonBkColor = SCREEN_BACKGROUND_COLOR;
	statusbox->EmbossDkColor = BLACK;
	statusbox->EmbossLtColor = BLACK;
	statusbox->TextColor0 = 65535;
	statusbox->TextColor1 = 31;
	statusbox->pFont = (void*)&MyNewFont;

	
	checkboxes = GOLCreateScheme();
	checkboxes->Color0 = 0;
	checkboxes->Color1 = 2016;
	checkboxes->ColorDisabled = 55070;
	checkboxes->CommonBkColor = SCREEN_BACKGROUND_COLOR;
	checkboxes->EmbossDkColor = 27501;
	checkboxes->EmbossLtColor = 50712;
	checkboxes->TextColor0 = 2016;
	checkboxes->TextColor1 = 2016;
	checkboxes->pFont = (void*)&SmallFonts;
	
}

void CreateLEDScreen(void)
{
	GOLFree();
	SetColor(SCREEN_BACKGROUND_COLOR);
	ClearDevice();

	// Initialize pointers for widgets on the screen
#if defined (PIC24FJ256DA210_DEV_BOARD)
	CHECKBOX *pOBJ_CHECKBOX_2_4;
	CHECKBOX *pOBJ_CHECKBOX_6_4;
#else
	CHECKBOX *pOBJ_CHECKBOX_1_4;
	CHECKBOX *pOBJ_CHECKBOX_2_4;
	CHECKBOX *pOBJ_CHECKBOX_3_4;
	CHECKBOX *pOBJ_CHECKBOX_4_4;
	CHECKBOX *pOBJ_CHECKBOX_5_4;
	CHECKBOX *pOBJ_CHECKBOX_6_4;
	CHECKBOX *pOBJ_CHECKBOX_7_4;
	CHECKBOX *pOBJ_CHECKBOX_8_4;
#endif
	
	STATICTEXT *pOBJ_STATICTEXT_TITLE;
	STATICTEXT *pOBJ_STATICTEXT_EVENT;

	pOBJ_STATICTEXT_TITLE = StCreate(OBJ_STATICTEXT_LAB3,
								LED_ST_STARTX,
								LED_ST_STARTY,
								LED_ST_STARTX+LED_ST_WIDTH,
								LED_ST_STARTY+LED_ST_HEIGHT,
								ST_CENTER_ALIGN|ST_DRAW|ST_FRAME,
							    "LED Control Panel",
							    statusbox);

	// Error Check
	if(pOBJ_STATICTEXT_TITLE==NULL){ //Fatal Error, Check for memory leak or heap size
		CreateError(0);
	}

#if defined (EXPLORER_16)

	// Create the widgets
	pOBJ_CHECKBOX_1_4 = CbCreate(OBJ_CHECKBOX_1,
								CB_1_STARTX,
								CB_1_STARTY,
								CB_1_STARTX + CB_WIDTH,
								CB_1_STARTY + CB_HEIGHT,
								CB_DRAW,
								" LED D3",
								checkboxes);
	
	// Error Check
	if(pOBJ_CHECKBOX_1_4==NULL){ //Fatal Error, Check for memory leak or heap size
		CreateError(0);
	}

	pOBJ_CHECKBOX_2_4 = CbCreate(OBJ_CHECKBOX_2,
								CB_2_STARTX,
								CB_2_STARTY,
								CB_2_STARTX + CB_WIDTH,
								CB_2_STARTY + CB_HEIGHT,	
								CB_DRAW,
								" LED D4",
								checkboxes);
	
	// Error Check
	if(pOBJ_CHECKBOX_2_4==NULL){ //Fatal Error, Check for memory leak or heap size
		CreateError(0);
	}

	pOBJ_CHECKBOX_3_4 = CbCreate(OBJ_CHECKBOX_3,
								CB_3_STARTX,
								CB_3_STARTY,
								CB_3_STARTX + CB_WIDTH,
								CB_3_STARTY + CB_HEIGHT,	
								CB_DRAW,
								" LED D5",
								checkboxes);
	
	// Error Check
	if(pOBJ_CHECKBOX_3_4==NULL){ //Fatal Error, Check for memory leak or heap size
		CreateError(0);
	}

	pOBJ_CHECKBOX_4_4 = CbCreate(OBJ_CHECKBOX_4,
								CB_4_STARTX,
								CB_4_STARTY,
								CB_4_STARTX + CB_WIDTH,
								CB_4_STARTY + CB_HEIGHT,	
								CB_DRAW,
								" LED D6",
								checkboxes);
	
	// Error Check
	if(pOBJ_CHECKBOX_4_4==NULL){ //Fatal Error, Check for memory leak or heap size
		CreateError(0);
	}

	pOBJ_CHECKBOX_5_4 = CbCreate(OBJ_CHECKBOX_5,
								CB_5_STARTX,
								CB_5_STARTY,
								CB_5_STARTX + CB_WIDTH,
								CB_5_STARTY + CB_HEIGHT,	
								CB_DRAW,
								" LED D7",
								checkboxes);

	// Error Check
	if(pOBJ_CHECKBOX_5_4==NULL){ //Fatal Error, Check for memory leak or heap size
		CreateError(0);
	}

	pOBJ_CHECKBOX_6_4 = CbCreate(OBJ_CHECKBOX_6,
								CB_6_STARTX,
								CB_6_STARTY,
								CB_6_STARTX + CB_WIDTH,
								CB_6_STARTY + CB_HEIGHT,	
								CB_DRAW,
								" LED D8",
								checkboxes);

	// Error Check
	if(pOBJ_CHECKBOX_6_4==NULL){ //Fatal Error, Check for memory leak or heap size
		CreateError(0);
	}

	pOBJ_CHECKBOX_7_4 = CbCreate(OBJ_CHECKBOX_7,
								CB_7_STARTX,
								CB_7_STARTY,
								CB_7_STARTX + CB_WIDTH,
								CB_7_STARTY + CB_HEIGHT,	
								CB_DRAW,
								" LED D9",
								checkboxes);

	// Error Check
	if(pOBJ_CHECKBOX_7_4==NULL){ //Fatal Error, Check for memory leak or heap size
		CreateError(0);
	}

	pOBJ_CHECKBOX_8_4 = CbCreate(OBJ_CHECKBOX_8,
								CB_8_STARTX,
								CB_8_STARTY,
								CB_8_STARTX + CB_WIDTH,
								CB_8_STARTY + CB_HEIGHT,	
								CB_DRAW,
								" LED D10",
								checkboxes);

	// Error Check
	if(pOBJ_CHECKBOX_8_4==NULL){ //Fatal Error, Check for memory leak or heap size
		CreateError(0);
	}

#else

	pOBJ_CHECKBOX_2_4 = CbCreate(OBJ_CHECKBOX_2,
								CB_2_STARTX,
								CB_2_STARTY,
								CB_2_STARTX + CB_WIDTH,
								CB_2_STARTY + CB_HEIGHT,	
								CB_DRAW,
								" LED D2",
								checkboxes);
	
	// Error Check
	if(pOBJ_CHECKBOX_2_4==NULL){ //Fatal Error, Check for memory leak or heap size
		CreateError(0);
	}
	
	pOBJ_CHECKBOX_6_4 = CbCreate(OBJ_CHECKBOX_4,
								CB_6_STARTX,
								CB_6_STARTY,
								CB_6_STARTX + CB_WIDTH,
								CB_6_STARTY + CB_HEIGHT,	
								CB_DRAW,
								" LED D4",
								checkboxes);

	// Error Check
	if(pOBJ_CHECKBOX_6_4==NULL){ //Fatal Error, Check for memory leak or heap size
		CreateError(0);
	}

#endif

	pOBJ_STATICTEXT_EVENT = StCreate(OBJ_STATICTEXT_EVENTS,
								BACK_BTN_STARTX - 30,
								BACK_BTN_STARTY,
								BACK_BTN_STARTX - 30 + BACK_BTN_WIDTH + 30,
								BACK_BTN_STARTY + BACK_BTN_HEIGHT,
								ST_CENTER_ALIGN|ST_DRAW,
							    "No Event",
							    checkboxes);

	// Error Check
	if(pOBJ_STATICTEXT_EVENT==NULL){ //Fatal Error, Check for memory leak or heap size
		CreateError(0);
	}




}

