/*****************************************************************************
* Microchip Graphics Library
* Graphics Display Designer (GDD) Template
*****************************************************************************
* Dependencies:    See INCLUDES section below
* Processor:       PIC24F, PIC24H, dsPIC, PIC32
* Compiler:       	MPLAB C30 V3.22, MPLAB C32 V1.05b
* Linker:          MPLAB LINK30, MPLAB LINK32
* Company:         Microchip Technology Incorporated
*
* Software License Agreement
*
* Copyright (c) 2010 Microchip Technology Inc.  All rights reserved.
* Microchip licenses to you the right to use, modify, copy and distribute
* Software only when embedded on a Microchip microcontroller or digital
* signal controller, which is integrated into your product or third party
* product (pursuant to the sublicense terms in the accompanying license
* agreement).  
*
* You should refer to the license agreement accompanying this Software
* for additional information regarding your rights and obligations.
*
* SOFTWARE AND DOCUMENTATION ARE PROVIDED AS IS WITHOUT WARRANTY OF ANY
* KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY WARRANTY
* OF MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR
* PURPOSE. IN NO EVENT SHALL MICROCHIP OR ITS LICENSORS BE LIABLE OR
* OBLIGATED UNDER CONTRACT, NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION,
* BREACH OF WARRANTY, OR OTHER LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT
* DAMAGES OR EXPENSES INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL,
* INDIRECT, PUNITIVE OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA,
* COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY
* CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF),
* OR OTHER SIMILAR COSTS.
*
* Author               Date        Comment
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*
*****************************************************************************/
#ifndef	_GDD_SCREENS_H_
#define	 _GDD_SCREENS_H_

//void GDDDemoCreateFirstScreen(void);
//void GDDDemoGOLMsgCallback(WORD objMsg, OBJ_HEADER* pObj, GOL_MSG* pMsg);
//void GDDDemoGOLDrawCallback(void);
//void GDDDemoNextScreen(void);

//extern const FONT_FLASH MS_Sans_Serif_10_bold;
//extern const FONT_FLASH MS_Sans_Serif_8_bold;

extern const FONT_FLASH SmallFonts;
extern const FONT_FLASH MyNewFont;

extern BITMAP_FLASH Rotate_Icon;
extern BITMAP_FLASH Temp_Icon;
extern BITMAP_FLASH LED_Icon;
extern BITMAP_FLASH Rotate_Icon_Dark;
extern BITMAP_FLASH Temp_Icon_Dark;
extern BITMAP_FLASH LED_Icon_Dark;


#define OBJ_STATICTEXT_MAIN_0   20
#define OBJ_BUTTON_ROTATE_POT  	21
#define OBJ_BUTTON_CHECK_TEMP	22
#define OBJ_BUTTON_CONTROL_LED	23
#define OBJ_STATICTEXT_MAIN_1   24
#define OBJ_STATICTEXT_MAIN_2   25
#define OBJ_STATICTEXT_MAIN_3   26

#define OBJ_CHECKBOX_1			30
#define OBJ_CHECKBOX_2			31
#define OBJ_CHECKBOX_3			32
#define OBJ_CHECKBOX_4			33
#define OBJ_CHECKBOX_5			34
#define OBJ_CHECKBOX_6			35
#define OBJ_CHECKBOX_7			36
#define OBJ_CHECKBOX_8			37
#define OBJ_STATICTEXT_LAB3		38
#define OBJ_STATICTEXT_EVENTS	39

#define OBJ_BUTTON_BACK			60


#define SCREEN_BACKGROUND_COLOR RGB565CONVERT(15, 15, 15) 

/* ************************************************************ */ 
//	macros that defines the dimension of the objects in the screen
/* ************************************************************ */ 

// For the Main Screen Button Objects, we center the buttons on the screen
#define BTN_WIDTH 				60
#define BTN_HEIGHT 				60
#define BTN_RADIUS 				15
#define POT_STARTX				((GetMaxX() - (BTN_WIDTH*3))/4)
#define POT_STARTY				((GetMaxY() - BTN_HEIGHT) >> 1)
#define TEMP_STARTX				((POT_STARTX*2) + BTN_WIDTH)
#define TEMP_STARTY				(POT_STARTY)
#define LED_STARTX				((POT_STARTX*3) + (BTN_WIDTH*2))
#define LED_STARTY				(POT_STARTY)

// For the Main Screen Static Text Object, we center the static text on the area of
// the screen above the buttons
#define ST_WIDTH 				(GetMaxX())
#define ST_HEIGHT 				40
#define ST_STARTX				0
#define ST_STARTY				((POT_STARTY - ST_HEIGHT) >> 1)


// For the LED Control Screen 
#define LED_ST_WIDTH 			(GetMaxX())
#define LED_ST_HEIGHT 			40
#define LED_ST_STARTX			0
#define LED_ST_STARTY			((POT_STARTY - ST_HEIGHT) >> 1)

// Back Button location and dimensions
#define BACK_BTN_WIDTH 			60
#define BACK_BTN_HEIGHT 		30
#define BACK_BTN_RADIUS 		0
#define BACK_BTN_STARTX			((GetMaxX() - BACK_BTN_WIDTH) >> 1)
#define BACK_BTN_STARTY			((GetMaxY() - BACK_BTN_HEIGHT))

// Locate the Check Boxes within the area covered by the Title and the back button
#define VERT_MARGIN				4					// margins of the check boxes from each other
#define HORZ_MARGIN				(GetMaxX()/5)
#define CB_WIDTH 				(HORZ_MARGIN + (HORZ_MARGIN>>1))
#define CB_HEIGHT				((GetMaxY() - (BACK_BTN_HEIGHT + LED_ST_STARTY + LED_ST_HEIGHT + (VERT_MARGIN*5)))/4)  
#define CB_1_STARTX				(HORZ_MARGIN)	
#define CB_1_STARTY				(LED_ST_STARTY + LED_ST_HEIGHT + VERT_MARGIN)	
#define CB_2_STARTX				(CB_1_STARTX)	
#define CB_2_STARTY				(CB_1_STARTY + CB_HEIGHT + VERT_MARGIN)	
#define CB_3_STARTX				(CB_1_STARTX)	
#define CB_3_STARTY				(CB_2_STARTY + CB_HEIGHT + VERT_MARGIN)	
#define CB_4_STARTX				(CB_1_STARTX)	
#define CB_4_STARTY				(CB_3_STARTY + CB_HEIGHT + VERT_MARGIN)	

#define CB_5_STARTX				(HORZ_MARGIN*3)	
#define CB_5_STARTY				(CB_1_STARTY)	
#define CB_6_STARTX				(CB_5_STARTX)	
#define CB_6_STARTY				(CB_5_STARTY  + CB_HEIGHT + VERT_MARGIN)	
#define CB_7_STARTX				(CB_5_STARTX)	
#define CB_7_STARTY				(CB_6_STARTY  + CB_HEIGHT + VERT_MARGIN)	
#define CB_8_STARTX				(CB_5_STARTX)	
#define CB_8_STARTY				(CB_7_STARTY  + CB_HEIGHT + VERT_MARGIN)	


	
void CreateSchemes(void);
void CreateMenuScreen(void);
void CreatePotentiometerScreen(void);
void CreateTemperatureScreen(void);
void CreateLEDScreen(void);

#endif
