/*****************************************************************************
* Microchip Graphics Library
* Graphics Display Designer (GDD) Template
*****************************************************************************
* Dependencies:    See INCLUDES section below
* Processor:       PIC24F, PIC24H, dsPIC, PIC32
* Compiler:       	MPLAB C30 V3.22, MPLAB C32 V1.05b
* Linker:          MPLAB LINK30, MPLAB LINK32
* Company:         Microchip Technology Incorporated
*
* Software License Agreement
*
* Copyright (c) 2010 Microchip Technology Inc.  All rights reserved.
* Microchip licenses to you the right to use, modify, copy and distribute
* Software only when embedded on a Microchip microcontroller or digital
* signal controller, which is integrated into your product or third party
* product (pursuant to the sublicense terms in the accompanying license
* agreement).  
*
* You should refer to the license agreement accompanying this Software
* for additional information regarding your rights and obligations.
*
* SOFTWARE AND DOCUMENTATION ARE PROVIDED AS IS WITHOUT WARRANTY OF ANY
* KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY WARRANTY
* OF MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR
* PURPOSE. IN NO EVENT SHALL MICROCHIP OR ITS LICENSORS BE LIABLE OR
* OBLIGATED UNDER CONTRACT, NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION,
* BREACH OF WARRANTY, OR OTHER LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT
* DAMAGES OR EXPENSES INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL,
* INDIRECT, PUNITIVE OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA,
* COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY
* CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF),
* OR OTHER SIMILAR COSTS.
*
* Author               Date        Comment
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*Kristy Seymour		4/20/10		Generated by GDD Beta version2.0.1
*								Modified to suit demo application
*****************************************************************************/
#include "Graphics\Graphics.h"
#include "MainGFXDemo.h"

// Strings
GOL_SCHEME* meterscheme;
GOL_SCHEME* statusbox;


static const XCHAR DoLab2[] 	= "Follow your manual for Lab 2.";


//  Create Error Flash (Blue Screen)
void CreateError(char* string)
{
	// Blue Screen Error
	SetColor(119);
	ClearDevice();
	SetColor(-1);
	
	// Flash Error Message
	if(string == NULL)
		{OutTextXY(0, 0, "Runtime Error.");}
	else{OutTextXY(0,0, string);}
	while(1);
}

void CreateSchemes(void)
{

	if(statusbox != NULL) free(statusbox);
		statusbox = GOLCreateScheme();
		statusbox->Color0 = 0;
		statusbox->Color1 = 64969;
		statusbox->ColorDisabled = 55070;
		statusbox->CommonBkColor = SCREEN_BACKGROUND_COLOR;
		statusbox->EmbossDkColor = 0;
		statusbox->EmbossLtColor = 0;
		statusbox->TextColor0 = 65535;
		statusbox->TextColor1 = 31;
		statusbox->pFont = (void*)&MyNewFont;
	
	if(meterscheme != NULL) free(meterscheme);
		meterscheme = GOLCreateScheme();
		meterscheme->Color0 = 0;
		meterscheme->Color1 = 2016;
		meterscheme->ColorDisabled = 55070;
		meterscheme->CommonBkColor = SCREEN_BACKGROUND_COLOR;
		meterscheme->EmbossDkColor = 0;
		meterscheme->EmbossLtColor = 0;
		meterscheme->TextColor0 = 2016;
		meterscheme->TextColor1 = 2016;
		meterscheme->pFont = (void*)&SmallFonts;	
	
}

void CreatePotentiometerScreen(void)
{
	GOLFree();
	SetColor(SCREEN_BACKGROUND_COLOR);
	ClearDevice();

// Initialize pointers for widgets on this screen
	METER *pOBJ_METER_1_1;				// Meter to show potentiometer value
	STATICTEXT *pOBJ_STATICTEXT_1_1;	// Screen Title
	
	static XCHAR Meter_Title[] = "10K Potentiometer";


	pOBJ_STATICTEXT_1_1 = StCreate(OBJ_STATICTEXT_LAB3,
								POT_ST_STARTX,
								POT_ST_STARTY,
								POT_ST_STARTX+POT_ST_WIDTH,
								POT_ST_STARTY+POT_ST_HEIGHT,
								ST_CENTER_ALIGN|ST_DRAW|ST_FRAME,
							    "View Potentiometer",statusbox);

	// Error Check
	if(pOBJ_STATICTEXT_1_1==NULL){ //Fatal Error, Check for memory leak or heap size
		CreateError(0);
	}

	// Meter to show potentiometer value
	pOBJ_METER_1_1 = MtrCreate(OBJ_METER_POT,
								MTR_STARTX,
								MTR_STARTY,
								MTR_STARTX + MTR_WIDTH,
								MTR_STARTY + MTR_HEIGHT,
								MTR_DRAW|MTR_RING,
								0,
								0,
								METERMAXVALUE,
								NULL,
								NULL,
								(XCHAR*)Meter_Title,
								meterscheme);
	
	//Error Check
	if(pOBJ_METER_1_1==NULL){ //Fatal Error, Check for memory leak or heap size
		CreateError(0);
	} 
	MtrSetTitleFont(pOBJ_METER_1_1,(void*)&SmallFonts);
	MtrSetValueFont(pOBJ_METER_1_1,(void*)&SmallFonts);

}



