/*****************************************************************************
* Microchip Graphics Library
* Graphics Display Designer (GDD) Template
*****************************************************************************
* Dependencies:    See INCLUDES section below
* Processor:       PIC24F, PIC24H, dsPIC, PIC32
* Compiler:       	MPLAB C30 V3.22, MPLAB C32 V1.05b
* Linker:          MPLAB LINK30, MPLAB LINK32
* Company:         Microchip Technology Incorporated
*
* Software License Agreement
*
* Copyright (c) 2010 Microchip Technology Inc.  All rights reserved.
* Microchip licenses to you the right to use, modify, copy and distribute
* Software only when embedded on a Microchip microcontroller or digital
* signal controller, which is integrated into your product or third party
* product (pursuant to the sublicense terms in the accompanying license
* agreement).  
*
* You should refer to the license agreement accompanying this Software
* for additional information regarding your rights and obligations.
*
* SOFTWARE AND DOCUMENTATION ARE PROVIDED AS IS WITHOUT WARRANTY OF ANY
* KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY WARRANTY
* OF MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR
* PURPOSE. IN NO EVENT SHALL MICROCHIP OR ITS LICENSORS BE LIABLE OR
* OBLIGATED UNDER CONTRACT, NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION,
* BREACH OF WARRANTY, OR OTHER LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT
* DAMAGES OR EXPENSES INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL,
* INDIRECT, PUNITIVE OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA,
* COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY
* CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF),
* OR OTHER SIMILAR COSTS.
*
* Author               Date        Comment
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*Kristy Seymour		4/20/10		Generated by GDD Beta version2.0.1
*								Modified to suit demo application
*****************************************************************************/
#include "Graphics\Graphics.h"
#include "MainGFXDemo.h"

// Strings
GOL_SCHEME* checkboxes;
GOL_SCHEME* meterscheme;
GOL_SCHEME* statusbox;
GOL_SCHEME* flatbuttons;
GOL_SCHEME* labelbox;


//  Create Error Flash (Blue Screen)
void CreateError(char* string)
{
	// Blue Screen Error
	SetColor(119);
	ClearDevice();
	SetColor(-1);
	
	// Flash Error Message
	if(string == NULL)
		{OutTextXY(0, 0, "Runtime Error.");}
	else{OutTextXY(0,0, string);}
	while(1);
}

void CreateSchemes(void)
{

	if(statusbox != NULL) free(statusbox);
		statusbox = GOLCreateScheme();
		statusbox->Color0 = 0;
		statusbox->Color1 = 64969;
		statusbox->ColorDisabled = 55070;
		statusbox->CommonBkColor = SCREEN_BACKGROUND_COLOR;
		statusbox->EmbossDkColor = 0;
		statusbox->EmbossLtColor = 0;
		statusbox->TextColor0 = 65535;
		statusbox->TextColor1 = 31;
		statusbox->pFont = (void*)&MyNewFont;
	
	if(flatbuttons != NULL) free(flatbuttons);	
		flatbuttons = GOLCreateScheme();
		flatbuttons->Color0 = RGB565CONVERT(0x75, 0x76, 0x75); //65500;
		flatbuttons->Color1 = 64969;
		flatbuttons->ColorDisabled = 55070;
		flatbuttons->CommonBkColor = SCREEN_BACKGROUND_COLOR;
		flatbuttons->EmbossDkColor = RGB565CONVERT(0x55, 0x56, 0x55); 
		flatbuttons->EmbossLtColor = RGB565CONVERT(0xCE, 0xCF, 0xCE);
		flatbuttons->TextColor0 = 8;
		flatbuttons->TextColor1 = 31;
		flatbuttons->pFont = (void*)&SmallFonts;
		
	if(labelbox != NULL) free(labelbox);
		labelbox = GOLCreateScheme();
		labelbox->Color0 = 0;
		labelbox->Color1 = 64969;
		labelbox->ColorDisabled = 55070;
		labelbox->CommonBkColor = SCREEN_BACKGROUND_COLOR;
		labelbox->EmbossDkColor = 0;
		labelbox->EmbossLtColor = 0;
		labelbox->TextColor0 = 65535;
		labelbox->TextColor1 = 31;
		labelbox->pFont = (void*)&SmallFonts;
		
	if(checkboxes != NULL) free(checkboxes);
		checkboxes = GOLCreateScheme();
		checkboxes->Color0 = 0;
		checkboxes->Color1 = 2016;
		checkboxes->ColorDisabled = 55070;
		checkboxes->CommonBkColor = SCREEN_BACKGROUND_COLOR;
		checkboxes->EmbossDkColor = 27501;
		checkboxes->EmbossLtColor = 50712;
		checkboxes->TextColor0 = 2016;
		checkboxes->TextColor1 = 2016;
		checkboxes->pFont = (void*)&SmallFonts;
	
	if(meterscheme != NULL) free(meterscheme);
		meterscheme = GOLCreateScheme();
		meterscheme->Color0 = 0;
		meterscheme->Color1 = 2016;
		meterscheme->ColorDisabled = 55070;
		meterscheme->CommonBkColor = SCREEN_BACKGROUND_COLOR;
		meterscheme->EmbossDkColor = 0;
		meterscheme->EmbossLtColor = 0;
		meterscheme->TextColor0 = 2016;
		meterscheme->TextColor1 = 2016;
		meterscheme->pFont = (void*)&SmallFonts;	
	
}

void CreateMenuScreen(void)
{

	// pointers to widgets on this screen
	BUTTON *pOBJ_BUTTON_POT = NULL;
	BUTTON *pOBJ_BUTTON_TEMP = NULL;
	BUTTON *pOBJ_BUTTON_LED = NULL;

	STATICTEXT *pOBJ_STATICTEXT_TITLE = NULL;
	STATICTEXT *pOBJ_STATICTEXT_POT = NULL;
	STATICTEXT *pOBJ_STATICTEXT_TEMP = NULL;
	STATICTEXT *pOBJ_STATICTEXT_LED = NULL;
	
	static XCHAR Btn_Label[] = "LED"; 
	
	//### <--- Lab5 Step 5:  Free the memory used by all widgets in the current active list ###
	GOLFree();
	
	SetColor(SCREEN_BACKGROUND_COLOR);
	ClearDevice();


	// Button to launch the potentiometer measurement screen
	pOBJ_BUTTON_POT = BtnCreate(OBJ_BUTTON_ROTATE_POT,
								POT_STARTX,
								POT_STARTY,
								POT_STARTX+BTN_WIDTH,
								POT_STARTY+BTN_HEIGHT,
								BTN_RADIUS,
								BTN_DRAW,
								(void*)&Rotate_Icon,
								NULL,
								NULL);
								
	// Button to launch the temperature monitoring screen
	pOBJ_BUTTON_TEMP = BtnCreate(OBJ_BUTTON_CHECK_TEMP,
								TEMP_STARTX,
								TEMP_STARTY,
								TEMP_STARTX+BTN_WIDTH,
								TEMP_STARTY+BTN_HEIGHT,
								BTN_RADIUS,
								BTN_DRAW,
								(void*)&Temp_Icon,
								NULL,
								NULL);

	// Button to launch LED Screen
	pOBJ_BUTTON_LED = BtnCreate(OBJ_BUTTON_CONTROL_LED,
								LED_STARTX,
								LED_STARTY,
								LED_STARTX+BTN_WIDTH,
								LED_STARTY+BTN_HEIGHT,
								BTN_RADIUS,
								BTN_DRAW,
								NULL, 
								(XCHAR*)Btn_Label,
								flatbuttons);

	// Error Checks
	if(pOBJ_BUTTON_POT==NULL){ //Fatal Error, Check for memory leak or heap size
		CreateError(0);
	}
	if(pOBJ_BUTTON_TEMP==NULL){//Fatal Error, Check for memory leak or heap size
		CreateError(0);
	}
	if(pOBJ_BUTTON_LED==NULL){//Fatal Error, Check for memory leak or heap size
		CreateError(0);
	}

	// Static Text to label the Potentiometer button
	pOBJ_STATICTEXT_POT = StCreate(OBJ_STATICTEXT_POT,
								POT_STARTX,
								POT_STARTY + BTN_HEIGHT + 5,
								POT_STARTX + BTN_WIDTH,
								POT_STARTY + (BTN_HEIGHT*2) + 5,
								ST_CENTER_ALIGN|ST_DRAW,
								"ROTATE\nPOT",
								labelbox);
	
	// Static Text to label the Temperature button
	pOBJ_STATICTEXT_TEMP = StCreate(OBJ_STATICTEXT_TEMP,
								TEMP_STARTX,
								TEMP_STARTY + BTN_HEIGHT + 5,
								TEMP_STARTX + BTN_WIDTH,
								TEMP_STARTY + (BTN_HEIGHT*2) + 5,
								ST_CENTER_ALIGN|ST_DRAW,
								"CHECK\nTEMP",
								labelbox);
	
	// Static Text to label the Temperature button
	pOBJ_STATICTEXT_LED = StCreate(OBJ_STATICTEXT_LED,
								LED_STARTX,
								LED_STARTY + BTN_HEIGHT + 5,
								LED_STARTX + BTN_WIDTH,
								LED_STARTY + (BTN_HEIGHT*2) + 5,
								ST_CENTER_ALIGN|ST_DRAW,
								"LED\nCONTROL",
								labelbox);

	// Static Text to label the screen
	pOBJ_STATICTEXT_TITLE = StCreate(OBJ_STATICTEXT_MAIN,
								ST_STARTX,
								ST_STARTY,
								ST_STARTX+ST_WIDTH,
								ST_STARTY+ST_HEIGHT,
								ST_CENTER_ALIGN|ST_DRAW|ST_FRAME,
								"Development Board Status",
								statusbox);
	
	// Error Checks
	if(pOBJ_STATICTEXT_POT==NULL){//Fatal Error, Check for memory leak or heap size
		CreateError(0);
	}
	if(pOBJ_STATICTEXT_TEMP==NULL){//Fatal Error, Check for memory leak or heap size
		CreateError(0);
	}
	if(pOBJ_STATICTEXT_TEMP==NULL){//Fatal Error, Check for memory leak or heap size
		CreateError(0);
	}
	if(pOBJ_STATICTEXT_TITLE==NULL){//Fatal Error, Check for memory leak or heap size
		CreateError(0);
	}
	
}

void CreateLEDScreen(void)
{

	//### <--- Lab5 Step 6:  Free the memory used by all widgets in the current active list ###
	GOLFree();

	SetColor(SCREEN_BACKGROUND_COLOR);
	ClearDevice();

	// Initialize pointers for widgets on the screen
#if defined (PIC24FJ256DA210_DEV_BOARD)
	CHECKBOX *pOBJ_CHECKBOX_2_4;
	CHECKBOX *pOBJ_CHECKBOX_6_4;
#else
	CHECKBOX *pOBJ_CHECKBOX_1_4;
	CHECKBOX *pOBJ_CHECKBOX_2_4;
	CHECKBOX *pOBJ_CHECKBOX_3_4;
	CHECKBOX *pOBJ_CHECKBOX_4_4;
	CHECKBOX *pOBJ_CHECKBOX_5_4;
	CHECKBOX *pOBJ_CHECKBOX_6_4;
	CHECKBOX *pOBJ_CHECKBOX_7_4;
	CHECKBOX *pOBJ_CHECKBOX_8_4;
#endif
	
	STATICTEXT *pOBJ_STATICTEXT_TITLE;
	BUTTON	   *pOBJ_BUTTON_BACK;

	pOBJ_STATICTEXT_TITLE = StCreate(OBJ_STATICTEXT_LAB3,
								LED_ST_STARTX,
								LED_ST_STARTY,
								LED_ST_STARTX+LED_ST_WIDTH,
								LED_ST_STARTY+LED_ST_HEIGHT,
								ST_CENTER_ALIGN|ST_DRAW|ST_FRAME,
							    "Development Board LED Control",
							    statusbox);

	// Error Check
	if(pOBJ_STATICTEXT_TITLE==NULL){ //Fatal Error, Check for memory leak or heap size
		CreateError(0);
	}

#if defined (EXPLORER_16)

	// Create the widgets
	pOBJ_CHECKBOX_1_4 = CbCreate(OBJ_CHECKBOX_1,
								CB_1_STARTX,
								CB_1_STARTY,
								CB_1_STARTX + CB_WIDTH,
								CB_1_STARTY + CB_HEIGHT,
								CB_DRAW,
								" LED D3",
								checkboxes);
	
	// Error Check
	if(pOBJ_CHECKBOX_1_4==NULL){ //Fatal Error, Check for memory leak or heap size
		CreateError(0);
	}

	pOBJ_CHECKBOX_2_4 = CbCreate(OBJ_CHECKBOX_2,
								CB_2_STARTX,
								CB_2_STARTY,
								CB_2_STARTX + CB_WIDTH,
								CB_2_STARTY + CB_HEIGHT,	
								CB_DRAW,
								" LED D4",
								checkboxes);
	
	// Error Check
	if(pOBJ_CHECKBOX_2_4==NULL){ //Fatal Error, Check for memory leak or heap size
		CreateError(0);
	}

	pOBJ_CHECKBOX_3_4 = CbCreate(OBJ_CHECKBOX_3,
								CB_3_STARTX,
								CB_3_STARTY,
								CB_3_STARTX + CB_WIDTH,
								CB_3_STARTY + CB_HEIGHT,	
								CB_DRAW,
								" LED D5",
								checkboxes);
	
	// Error Check
	if(pOBJ_CHECKBOX_3_4==NULL){ //Fatal Error, Check for memory leak or heap size
		CreateError(0);
	}

	pOBJ_CHECKBOX_4_4 = CbCreate(OBJ_CHECKBOX_4,
								CB_4_STARTX,
								CB_4_STARTY,
								CB_4_STARTX + CB_WIDTH,
								CB_4_STARTY + CB_HEIGHT,	
								CB_DRAW,
								" LED D6",
								checkboxes);
	
	// Error Check
	if(pOBJ_CHECKBOX_4_4==NULL){ //Fatal Error, Check for memory leak or heap size
		CreateError(0);
	}

	pOBJ_CHECKBOX_5_4 = CbCreate(OBJ_CHECKBOX_5,
								CB_5_STARTX,
								CB_5_STARTY,
								CB_5_STARTX + CB_WIDTH,
								CB_5_STARTY + CB_HEIGHT,	
								CB_DRAW,
								" LED D7",
								checkboxes);

	// Error Check
	if(pOBJ_CHECKBOX_5_4==NULL){ //Fatal Error, Check for memory leak or heap size
		CreateError(0);
	}

	pOBJ_CHECKBOX_6_4 = CbCreate(OBJ_CHECKBOX_6,
								CB_6_STARTX,
								CB_6_STARTY,
								CB_6_STARTX + CB_WIDTH,
								CB_6_STARTY + CB_HEIGHT,	
								CB_DRAW,
								" LED D8",
								checkboxes);

	// Error Check
	if(pOBJ_CHECKBOX_6_4==NULL){ //Fatal Error, Check for memory leak or heap size
		CreateError(0);
	}

	pOBJ_CHECKBOX_7_4 = CbCreate(OBJ_CHECKBOX_7,
								CB_7_STARTX,
								CB_7_STARTY,
								CB_7_STARTX + CB_WIDTH,
								CB_7_STARTY + CB_HEIGHT,	
								CB_DRAW,
								" LED D9",
								checkboxes);

	// Error Check
	if(pOBJ_CHECKBOX_7_4==NULL){ //Fatal Error, Check for memory leak or heap size
		CreateError(0);
	}

	pOBJ_CHECKBOX_8_4 = CbCreate(OBJ_CHECKBOX_8,
								CB_8_STARTX,
								CB_8_STARTY,
								CB_8_STARTX + CB_WIDTH,
								CB_8_STARTY + CB_HEIGHT,	
								CB_DRAW,
								" LED D10",
								checkboxes);

	// Error Check
	if(pOBJ_CHECKBOX_8_4==NULL){ //Fatal Error, Check for memory leak or heap size
		CreateError(0);
	}

#else

	pOBJ_CHECKBOX_2_4 = CbCreate(OBJ_CHECKBOX_2,
								CB_2_STARTX,
								CB_2_STARTY,
								CB_2_STARTX + CB_WIDTH,
								CB_2_STARTY + CB_HEIGHT,	
								CB_DRAW,
								" LED D2",
								checkboxes);
	
	// Error Check
	if(pOBJ_CHECKBOX_2_4==NULL){ //Fatal Error, Check for memory leak or heap size
		CreateError(0);
	}
	
	pOBJ_CHECKBOX_6_4 = CbCreate(OBJ_CHECKBOX_4,
								CB_6_STARTX,
								CB_6_STARTY,
								CB_6_STARTX + CB_WIDTH,
								CB_6_STARTY + CB_HEIGHT,	
								CB_DRAW,
								" LED D4",
								checkboxes);

	// Error Check
	if(pOBJ_CHECKBOX_6_4==NULL){ //Fatal Error, Check for memory leak or heap size
		CreateError(0);
	}

#endif

	// BACK button -- used to return user to main menu
	pOBJ_BUTTON_BACK = BtnCreate(OBJ_BUTTON_BACK,
								BACK_BTN_STARTX,
								BACK_BTN_STARTY,
								BACK_BTN_STARTX + BACK_BTN_WIDTH,
								BACK_BTN_STARTY + BACK_BTN_HEIGHT,
								BACK_BTN_RADIUS,
								BTN_DRAW,
								NULL,
								"BACK",
								checkboxes);

	// Error Check
	if(pOBJ_BUTTON_BACK==NULL){	//Fatal Error, Check for memory leak or heap size
		CreateError(0);	
	} 

}

void CreatePotentiometerScreen(void)
{
	//### <--- Lab5 Step 7:  Free the memory used by all widgets in the current active list ###
	GOLFree();

	SetColor(SCREEN_BACKGROUND_COLOR);
	ClearDevice();

// Initialize pointers for widgets on this screen
	BUTTON *pOBJ_BUTTON_1_1;			// Back Button
	METER *pOBJ_METER_1_1;				// Meter to show potentiometer value
	STATICTEXT *pOBJ_STATICTEXT_1_1;	// Screen Title
	
	static XCHAR Meter_Title[] = "10K Potentiometer";


	pOBJ_STATICTEXT_1_1 = StCreate(OBJ_STATICTEXT_LAB3,
								POT_ST_STARTX,
								POT_ST_STARTY,
								POT_ST_STARTX+POT_ST_WIDTH,
								POT_ST_STARTY+POT_ST_HEIGHT,
								ST_CENTER_ALIGN|ST_DRAW|ST_FRAME,
							    "View Potentiometer",statusbox);

	// Error Check
	if(pOBJ_STATICTEXT_1_1==NULL){ //Fatal Error, Check for memory leak or heap size
		CreateError(0);
	}

	// BACK button -- used to return user to main menu
	pOBJ_BUTTON_1_1 = BtnCreate(OBJ_BUTTON_BACK,
								BACK_BTN_STARTX,
								BACK_BTN_STARTY,
								BACK_BTN_STARTX + BACK_BTN_WIDTH,
								BACK_BTN_STARTY + BACK_BTN_HEIGHT,
								BACK_BTN_RADIUS,
								BTN_DRAW,
								NULL,
								"BACK",
								checkboxes);
	// Error Check
	if(pOBJ_BUTTON_1_1==NULL){	//Fatal Error, Check for memory leak or heap size
		CreateError(0);	
	} 

	// Meter to show potentiometer value
	pOBJ_METER_1_1 = MtrCreate(OBJ_METER_POT,
								MTR_STARTX,
								MTR_STARTY,
								MTR_STARTX + MTR_WIDTH,
								MTR_STARTY + MTR_HEIGHT,
								MTR_DRAW|MTR_RING,
								0,
								0,
								POTMETERMAXVALUE,
								NULL,
								NULL,
								(XCHAR*)Meter_Title,
								meterscheme);
	
	//Error Check
	if(pOBJ_METER_1_1==NULL){ //Fatal Error, Check for memory leak or heap size
		CreateError(0);
	} 
	MtrSetTitleFont(pOBJ_METER_1_1,(void*)&SmallFonts);
	MtrSetValueFont(pOBJ_METER_1_1,(void*)&SmallFonts);

}

void CreateTemperatureScreen(void)
{

	//### <--- Lab5 Step 8:  Free the memory used by all widgets in the current active list ###
	GOLFree();

	SetColor(SCREEN_BACKGROUND_COLOR);
	ClearDevice();

	// Initialize pointers for widgets on this screen
	STATICTEXT *pOBJ_STATICTEXT_1_3;	// Screen Title
	BUTTON *pOBJ_BUTTON_1_3;			// Back Button
	METER *pOBJ_METER_1_3;				// Meter to show temperature value

	static XCHAR Meter_Title[] = "Temperature";


	pOBJ_STATICTEXT_1_3 = StCreate(OBJ_STATICTEXT_LAB3,
								TEMP_ST_STARTX,
								TEMP_ST_STARTY,
								TEMP_ST_STARTX+TEMP_ST_WIDTH,
								TEMP_ST_STARTY+TEMP_ST_HEIGHT,
								ST_CENTER_ALIGN|ST_DRAW|ST_FRAME,
							    "View Temperature",statusbox);

	// Error Check
	if(pOBJ_STATICTEXT_1_3==NULL){ //Fatal Error, Check for memory leak or heap size
		CreateError(0);
	}

	// BACK button -- used to return user to main menu
	pOBJ_BUTTON_1_3 = BtnCreate(OBJ_BUTTON_BACK,
								BACK_BTN_STARTX,
								BACK_BTN_STARTY,
								BACK_BTN_STARTX + BACK_BTN_WIDTH,
								BACK_BTN_STARTY + BACK_BTN_HEIGHT,
								BACK_BTN_RADIUS,
								BTN_DRAW,
								NULL,
								"BACK",
								checkboxes);
	// Error Check
	if(pOBJ_BUTTON_1_3==NULL){	//Fatal Error, Check for memory leak or heap size
		CreateError(0);	
	} 

	// Meter to show potentiometer value
	pOBJ_METER_1_3 = MtrCreate(OBJ_METER_POT,
								MTR_STARTX,
								MTR_STARTY,
								MTR_STARTX + MTR_WIDTH,
								MTR_STARTY + MTR_HEIGHT,
								MTR_DRAW|MTR_RING,
								0,
								0,
								TEMPMETERMAXVALUE,
								NULL,
								NULL,
								(XCHAR*)Meter_Title,
								meterscheme);
	
	//Error Check
	if(pOBJ_METER_1_3==NULL){ //Fatal Error, Check for memory leak or heap size
		CreateError(0);
	} 
	MtrSetTitleFont(pOBJ_METER_1_3,(void*)&SmallFonts);
	MtrSetValueFont(pOBJ_METER_1_3,(void*)&SmallFonts);


}


void UpdatePotentiometer(void)
{
	METER *pMtr = (METER*)GOLFindObject(OBJ_METER_POT);
	WORD mtrValue;
	
	mtrValue = ADCGetPot();
	if (mtrValue != MtrGetVal(pMtr))
	{
		MtrSetVal(pMtr, mtrValue);
		SetState(pMtr, MTR_DRAW_UPDATE);
	}	

}	

void UpdateTemperature(void)
{
	METER *pMtr = (METER*)GOLFindObject(OBJ_METER_POT);
	WORD mtrValue;

	mtrValue = ADCGetPot()/10;
	if (mtrValue != MtrGetVal(pMtr))
	{
		MtrSetVal(pMtr, mtrValue);
		SetState(pMtr, MTR_DRAW_UPDATE);
	}	

}	

