/*********************************************************************
 * Microchip Graphic Library Demo Application
 * The header file joins all header files used in the project.
 *********************************************************************
 * FileName:        MainDemo.h
 * Dependencies:    See INCLUDES section below
 * Processor:       PIC24/PIC32
 * Compiler:        C30/C32
 * Company:         Microchip Technology, Inc.
 *
 * Software License Agreement
 *
 * Copyright  2007 Microchip Technology Inc.  All rights reserved.
 * Microchip licenses to you the right to use, modify, copy and distribute
 * Software only when embedded on a Microchip microcontroller or digital
 * signal controller, which is integrated into your product or third party
 * product (pursuant to the sublicense terms in the accompanying license
 * agreement).  
 *
 * You should refer to the license agreement accompanying this Software
 * for additional information regarding your rights and obligations.
 *
 * SOFTWARE AND DOCUMENTATION ARE PROVIDED AS IS WITHOUT WARRANTY OF ANY
 * KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY WARRANTY
 * OF MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR
 * PURPOSE. IN NO EVENT SHALL MICROCHIP OR ITS LICENSORS BE LIABLE OR
 * OBLIGATED UNDER CONTRACT, NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION,
 * BREACH OF WARRANTY, OR OTHER LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT
 * DAMAGES OR EXPENSES INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL,
 * INDIRECT, PUNITIVE OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA,
 * COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY
 * CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF),
 * OR OTHER SIMILAR COSTS.
 *
 * Author               Date        Comment
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Anton Alkhimenok
 * Paolo Tamayo         03/10/08    ...
 ********************************************************************/
#ifndef _MAINDEMO_H
    #define _MAINDEMO_H

////////////////////////////// INCLUDES //////////////////////////////
    #ifdef __PIC32MX
        #include <plib.h>
        #include <p32xxxx.h>
    #else
        #include <p24Fxxxx.h>
    #endif
    #include <stdlib.h>
    #include <string.h>
    #include <stdio.h>

    #include "GenericTypeDefs.h"
    #include "Graphics\Graphics.h"
    #include "SST25VF016.h"
    #include "TouchScreen.h"
    #include "Screens.h"
    #include "Icons.h"    
 	
/////////////////////////////////////////////////////////////////////////////
//                            ADDITIONAL COLORS USED
/////////////////////////////////////////////////////////////////////////////

#define GRAY20      			RGB565CONVERT(51, 51, 51)
#define GRAY40      			RGB565CONVERT(102, 102, 102)
#define GRAY80      			RGB565CONVERT(204, 204, 204)
#define GRAY90      			RGB565CONVERT(229, 229, 229)
#define GRAY95      			RGB565CONVERT(242, 242, 242)
#define RED4        			RGB565CONVERT(139, 0, 0)
#define FIREBRICK1  			RGB565CONVERT(255, 48, 48)
#define DARKGREEN   			RGB565CONVERT(0, 100, 0)
#define PALEGREEN   			RGB565CONVERT(152, 251, 152)
#define LIGHTYELLOW 			RGB565CONVERT(238, 221, 130)
#define GOLD        			RGB565CONVERT(255, 215, 0)
#define DARKORANGE  			RGB565CONVERT(255, 140, 0)

/////////////////////////////////////////////////////////////////////////////
//                            GLOBAL DEFINES
/////////////////////////////////////////////////////////////////////////////

#define INCREMENT_TIME	   1			// used to increment the time value
#define DECREMENT_TIME	   0			// used to decrement the time value

#define TRUE			   1			// generic 1 (TRUE) constant
#define FALSE			   0			// generic 0 (FASLE) constant

#define LED_OFF			   0			// constant to disable LED
#define LED_ON			   1			// constant to enable LED 

/////////////////////////////////////////////////////////////////////////////
//                            SCREEN STATES
/////////////////////////////////////////////////////////////////////////////

typedef enum {
    CREATE_MAINMENU = 0,
    SHOW_MAINMENU,
    CREATE_POTENTIOMETER,
    SHOW_POTENTIOMETER,
	CREATE_TEMPERATURE,
	SHOW_TEMPERATURE,
    CREATE_LED,
    SHOW_LED,
} SCREEN_STATES;


#endif
