/*********************************************************************
 *
 *  Main Application Entry Point for the Graphics demo
 *
 *********************************************************************
 * FileName:        MainDemo.c
 * Dependencies:    Graphics.h
 * Processor:       PIC32
 * Compiler:        Microchip C32 v1.05 or higher
 *					
 * Company:         Microchip Technology, Inc.
 *
 * Copyright (C) 2010 Microchip Technology Inc.  All rights
 * reserved.
 *
 *
 * THE SOFTWARE AND DOCUMENTATION ARE PROVIDED "AS IS" WITHOUT
 * WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT
 * LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT SHALL
 * MICROCHIP BE LIABLE FOR ANY INCIDENTAL, SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF
 * PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR SERVICES, ANY CLAIMS
 * BY THIRD PARTIES (INCLUDING BUT NOT LIMITED TO ANY DEFENSE
 * THEREOF), ANY CLAIMS FOR INDEMNITY OR CONTRIBUTION, OR OTHER
 * SIMILAR COSTS, WHETHER ASSERTED ON THE BASIS OF CONTRACT, TORT
 * (INCLUDING NEGLIGENCE), BREACH OF WARRANTY, OR OTHERWISE.
 *
 * Author              Date         Comment
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Aseem swalah		   06/17/10		...
 ********************************************************************/
// Include functions specific to this stack application
#include "MainDemo.h"
#include "GraphicsDemo.h"

#pragma config FPLLODIV = DIV_1, FPLLMUL = MUL_20, FPLLIDIV = DIV_2, FWDTEN = OFF, FPBDIV = DIV_1, POSCMOD = XT, FNOSC = PRIPLL, CP = OFF
#pragma config FMIIEN = OFF, FETHIO = OFF	// external PHY in RMII/alternate configuration
		
		
// Private helper functions.
// These may or may not be present in all applications.
static void InitializeBoard(void);

// Main application entry Point
int main(void)
{	    
	// Initialize application specific hardware
	InitializeBoard();
	
	//Display Initialize.
	DisplayInit();
	
	//Create the graphics objects
	CreateGraphics();
	
    while(1)
    {
		DisplayTask();
	}
	
	return 1;
}

/****************************************************************************
  Function:
    static void InitializeBoard(void)

  Description:
    This routine initializes the hardware using definitions
    in HardwareProfile.h 

  Precondition:
    None

  Parameters:
    None - None

  Returns:
    None

  Remarks:
    None
  ***************************************************************************/
static void InitializeBoard(void)
{	
	LATBbits.LATB15 = 0;
    TRISBbits.TRISB15 = 0;
    
	// Enable multi-vectored interrupts
	INTEnableSystemMultiVectoredInt();
	
	// Enable optimal performance
	SYSTEMConfigPerformance(GetSystemClock());

	TurnLEDAllOff();
    SetLEDDirection();

    DelayMs(100);
    TurnLEDOn(LED_2);
    DelayMs(100);
    TurnLEDOn(LED_3);
    DelayMs(100);
    TurnLEDOn(LED_4);
    DelayMs(100);
    TurnLEDOn(LED_10);
    DelayMs(100);
    TurnLEDOn(LED_5);
    DelayMs(100);

    TurnLEDAllOff();
	
    CPLDInitialize();
    CPLDSetGraphicsConfiguration(GRAPHICS_HW_CONFIG);
    CPLDSetSPIFlashConfiguration(SPI_FLASH_CHANNEL);
}


